/***************************************************************************
                      blocklist.cpp  -  list of blocks
                     ----------------------------------
    begin                : Sat Jul 29 2000
    copyright            : (C) 2000 by Ladislav Slezk
    email                : slezak@penguin.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "blocklist.h"
#include "wavblock.h"
#include "mp3block.h"
#include "dialogresize.h"
#include "utils.h"

#include <gtk--/main.h>
#include <gnome--/app.h>
#include <libgnome/gnome-i18n.h>

#include <gnome--/dialog.h>
#include <unistd.h>

BlockList::BlockList(){   // reset list state
	playing = false;
	paused = false;
	actual = 0;
	actualIndex = 0;
	Sdevice = 0;
	chg = false;
	cutting = false;
	export_progress = 0;
}

BlockList::BlockList(const BlockList &src) {    // create copy of BlockList
	playing = false;
	paused = false;
	actual = 0;
	actualIndex = 0;
	Sdevice = 0;
	chg = false;
	cutting = false;
	export_progress = 0;
	
	unsigned int index = 0;
	MP3block *mp3b, *newmp3;
	WAVblock *wavb, *newwav;
	
	// create copies of sound blocks
	// use RTTI do get type of sound block
	while (index < src.blocks.size()) {
	  mp3b = dynamic_cast<MP3block*>(src.blocks[index]);
	
	  if (mp3b) {                             // is it MP3block?
	    newmp3 = new MP3block(*mp3b);
	    add_block(dynamic_cast<SoundBlock*>(newmp3));
	  }
	  else {
  	  wavb = dynamic_cast<WAVblock*>(src.blocks[index]);  // is it WAVblock?
   	  if (wavb) {
  	    newwav = new WAVblock(*wavb);
  	    add_block(dynamic_cast<SoundBlock*>(newwav));
   	  }
   	  else
   	    cerr << "Unknown class of SoundBlock???" << endl;
	  }
	  index++;
	}
}

BlockList::~BlockList(){
  // delete blocks
  while (blocks.size() > 0) {
    delete blocks[0];
    blocks.erase(blocks.begin());
  }
}

void BlockList::play_list(SoundDevice *sd, string mpg123_path, int from_ms = -1, int to_ms = -1) {
  if (playing) return;

#ifdef DEBUG_OUTPUT
  cerr << "BlockList:  from_ms = " << from_ms << "      to_ms = " << to_ms << endl;
#endif

  play_to_ms = to_ms;
  play_from_ms = from_ms;

  totalms = 0;
  totaldone = 0;
  mpg123_decoder = mpg123_path;

  int i = blocks.size();
  int index = 0;

  if (i == 0) {            // list is empty
  	play_done.emit();
  	return;
  }

  playing = true;
  Sdevice = sd;

/////////////
  int total = blocks[0]->block_length();
  int numblocks = static_cast<int>(blocks.size());
  ms1 = 0;
  ms2 = 0;

  if (from_ms < 0 && to_ms < 0) {		// play all blocks
    firstblock = 0;
    lastblock = numblocks - 1;
    index = firstblock;

    // initialize playing
    while (index < lastblock) {
      blocks[index]->init_play();
      index++;
    }

    // set actual block to first block
    index = firstblock;
    actualIndex = firstblock;

    // play first block
    actual = blocks[index];
    actual->play_block(sd, mpg123_decoder);
    return;
  }

  if (from_ms > 0 && to_ms < 0) {		// play end
    while (total < from_ms && index < numblocks - 1) {
      ms1 = total;
      index++;
      total += blocks[index]->block_length();
    }
    firstblock = index;
    lastblock = numblocks - 1;
    index = firstblock;

    // initialize playing
    while (index < lastblock) {
      blocks[index]->init_play();
      index++;
    }

    // set actual block to first block
    index = firstblock;
    actualIndex = firstblock;

    // play first block
    actual = blocks[index];

    totaldone = ms1;
    play_block_done.emit(from_ms);
    actual->play_block(sd, mpg123_decoder, from_ms - ms1);
    return;
  }

  if (from_ms < 0 && to_ms > 0) {		// play start

    while (total < to_ms && index < numblocks - 1) {
      ms1 = total;
      index++;
      total += blocks[index]->block_length();
    }
    firstblock = 0;
    lastblock = index;
    index = firstblock;

#ifdef DEBUG_OUTPUT
    cerr << "BLockList playstart: first: " << firstblock << "    last: " << lastblock << endl;
#endif

    // initialize playing
    while (index < lastblock) {
      blocks[index]->init_play();
      index++;
    }

    // set actual block to first block
    index = firstblock;
    actualIndex = firstblock;

    // play first block
    actual = blocks[index];

    totaldone = 0;

    if (firstblock == lastblock)
      actual->play_block(sd, mpg123_decoder, -1, to_ms);
    else
      actual->play_block(sd, mpg123_decoder);

    return;
  }


}

void BlockList::stop_list() {
  if (playing) {
  	playing = false;
  	if (paused) {
  	  blocks[actualIndex]->unpause_block();
  	  paused = false;
  	}
  	blocks[actualIndex]->stop_block();
  	play_done.emit();
  }
}


void BlockList::pause_list() {
  if (playing) {
  	paused = true;
  	actual->pause_block();
  }
}

void BlockList::unpause_list() {
  if (playing) {
  	paused = false;
  	actual->unpause_block();
  }
}


void BlockList::display_blocks(Gdk_Pixmap *pix, Gdk_GC gc) {
	int pixw = pix->width();      // pixmap width
	int pixh = pix->height();     // pixmap height

  int milisecT = total_ms();     // total project time

  int i = blocks.size();        // number of blocks
  int index = 0;
  double milisec;
  double w;
  double xpos = 0.0;

  while (index < i) {
    milisec = static_cast<double>(blocks[index]->num_samples()) / (blocks[index]->get_frequency()) * 1e3;
    w = milisec / milisecT * pixw;

    // draw block to pixmap
    blocks[index]->display(pix, gc, static_cast<int>(xpos), pixh, static_cast<int>(w));

    xpos += w;
    index++;
  }
}

int BlockList::total_ms() {
  int i = blocks.size();
  int index = 0;
  double milisec = 0;

  while (index < i) {
    milisec +=  static_cast<double>(blocks[index]->num_samples()) / (blocks[index]->get_frequency()) * 1e3;
    index++;
  }

  return static_cast<int>(milisec);
}

void  BlockList::add_block(SoundBlock *sb) {
  sb->block_done.connect(slot(this, &BlockList::block_play_done));
//  sb->playedMiliSec.connect(slot(this, &BlockList::blockPlayedMiliSecs));
  sb->block_changed.connect(slot(this, &BlockList::sound_block_changed));
  sb->export_done.connect(slot(this, &BlockList::block_export_done));

  blocks.push_back(sb);

  // block list changed
  chg = true;
  block_list_changed.emit();
}

bool BlockList::is_playing() {
	return playing;
}

bool BlockList::is_paused() {
	return paused;
}

int BlockList::num_blocks() {
  return blocks.size();
}

SoundBlock *BlockList::block_at(int index) {
	if (unsigned(index) < blocks.size())
		return blocks[index];
	else
		return 0;
}

void BlockList::remove_block(int pos) {
	if (unsigned(pos) < blocks.size()) {
		blocks.erase(blocks.begin() + pos);
		chg = true;
    if (!cutting) block_list_changed.emit();
	}
}

void BlockList::block_play_done() {
	totaldone += actual->block_length();
	totalms = totaldone;

	if (play_from_ms > 0) {
  	play_block_done.emit(actual->block_length() - play_from_ms);
  	play_from_ms = 0;
  }
	else
  	play_block_done.emit(actual->block_length());
	
	actualIndex++;
	
//	if (unsigned(actualIndex) < blocks.size() && playing) {
	if (actualIndex <= lastblock && playing) {
		actual = blocks[actualIndex];
		
	  if (play_to_ms > 0 && actualIndex == lastblock) {
#ifdef DEBUG_OUTPUT
   cerr << "play start: last block, play_to_ms= " << play_to_ms << "   totaldone=" << totaldone << endl;
#endif

	    actual->play_block(Sdevice, mpg123_decoder, -1, play_to_ms - totaldone);
    }
	  else
  	  actual->play_block(Sdevice, mpg123_decoder);
	}
	else {
		Sdevice->close_device();
		play_done.emit();
		playing = false;
	}
}

void BlockList::scanned() {
	scan_done.emit();
}

bool BlockList::block_to_end(int pos) {
 	if (unsigned(pos) < blocks.size() - 1 && pos >= 0) {
		SoundBlock *tmp = blocks[pos];
		
		blocks[pos] = blocks[pos + 1];
		blocks[pos + 1] = tmp;
		
		chg = true;
    block_list_changed.emit();
		return true;
	}
	return false;
}

bool BlockList::block_to_begin(int pos) {
 	if (unsigned(pos) < blocks.size() && pos > 0) {
		SoundBlock *tmp = blocks[pos];
		
		blocks[pos] = blocks[pos - 1];
		blocks[pos - 1] = tmp;
		
		chg = true;
    block_list_changed.emit();
		return true;
	}
	return false;
}

bool BlockList::save_BlockList_to_file(string fname, bool ask_rewrite = true) {
  ofstream os;

  if (ask_rewrite) {    // acknowlegment required?
    ifstream is;
    is.open(fname.c_str(), ios::binary | ios::in);

    if (is.is_open()) {
      string question;
      question = _("File ") + fname + _(" already exists.\nRewrite file?");
  		Gnome::Dialog *q = Gnome::Dialogs::ok_cancel_modal(question);
  		if (q->run_and_close() == 1) return false;
    }
  }

  os.open(fname.c_str(), ios::binary | ios::out);			// open output stream

  if (!os.is_open()) {
    cerr << "Not opened!!" << endl;
    return false;
  }

  os << "Quickedit";

  int sz = blocks.size();
  int i = 0;
  string dir(fname);

  dir.erase(dir.begin() + dir.rfind('/'), dir.end());

  while (i < sz) {
    blocks[i]->save_block_to_stream(os, dir);
    i++;
  }

  fileName = fname;
  chg = false;        // all changed saved

  return true;
}

bool BlockList::load_BlockList_from_file(string fname) {
  ifstream is;
  is.open(fname.c_str(), ios::binary | ios::in);

  if (!is.is_open()) {
    string question = _("Error opening file ") + fname;
		Gnome::Dialog *q = Gnome::Dialogs::warning(question);
		q->run_and_close();
	  fileName = "";
		return false;
  }

  int dirpos = fname.rfind('/');         // change directory to projects's dir (file names  are relative to it)
  if (dirpos >= 0) {
    string dirName(fname, 0, dirpos);
    chdir(dirName.c_str());
#ifdef DEBUG_OUTPUT
    cerr << dirName << endl;
    cerr << chdir(dirName.c_str());
#endif
  }

	string q;
	int num = 9;
	
	while (num > 0) {
		char x;
  	is.read(&x, sizeof(x));
  	q = q + x;
  	num--;
  }

#ifdef DEBUG_OUTPUT
  cerr << q << endl;
#endif

  if (q != string("Quickedit")) {
    string question = _("Bad header in file ") + fname + _("\nProbably not Quickedit project file.");
		Gnome::Dialog *q = Gnome::Dialogs::error(question);
		q->run_and_close();
	  fileName = "";
		return false;
	}

  // delete old project
  blocks.erase(blocks.begin(), blocks.end());
	
  while (!is.eof()) {
    char c;
  	is.read(&c, sizeof(c));

  	if (is.eof()) {
 		  fileName = fname;
      chg = false;                // project is loaded, no change
    	return true;
  	}
  	
    switch (c) {
      case('W') : {           // WAV block
        WAVblock *wb = new WAVblock;

        if (wb->load_block_from_stream(is)) {
          // add block to list without emitting signal block_list_changed
          wb->block_done.connect(slot(this, &BlockList::block_play_done));
//          wb->playedMiliSec.connect(slot(this, &BlockList::blockPlayedMiliSecs));
          wb->block_changed.connect(slot(this, &BlockList::sound_block_changed));
          wb->export_done.connect(slot(this, &BlockList::block_export_done));

          blocks.push_back(wb);
        }
        else
          delete wb;        // error loading block

        break;
      }

      case('M') : {           // MPEG block
        MP3block *mb = new MP3block;

        if (mb->load_block_from_stream(is)) {
          // add block to list without emitting signal block_list_changed
          mb->block_done.connect(slot(this, &BlockList::block_play_done));
//          mb->playedMiliSec.connect(slot(this, &BlockList::blockPlayedMiliSecs));
          mb->block_changed.connect(slot(this, &BlockList::sound_block_changed));
          mb->export_done.connect(slot(this, &BlockList::block_export_done));

          blocks.push_back(mb);
        }
        else
          delete mb;        // error loading block

        break;
      }

      default : {             // unknown block
        string question = _("File format error: Unknown block type");
    		Gnome::Dialog *q = Gnome::Dialogs::error(question);
    		q->run_and_close();
   		  fileName = "";

        return false;
      }
    }
  }

  fileName = fname;
  chg = false;                // project is loaded, no change

  return true;
}

//void BlockList::blockPlayedMiliSecs(int ms) {
//  totalms = totaldone + ms;
//}

void BlockList::clear() {
  if (playing) stop_list();

  blocks.erase(blocks.begin(), blocks.end());
}

void BlockList::select(int from_ms, int to_ms) {
  int totalms = total_ms();

  if (from_ms > totalms) return;
  if (to_ms > totalms) to_ms = totalms;

  int total = blocks[0]->block_length();
  unsigned int index = 0;
  unsigned int numblocks = blocks.size();
  int ms1 = 0, ms2;
  unsigned int firstblock, lastblock;

  while (total < from_ms && index < numblocks) {
    ms1 = total;
    index++;
    total += blocks[index]->block_length();
  }

  firstblock = index;
  ms2 = ms1;

  while (total < to_ms && index < numblocks - 1) {
    ms2 = total;
    index++;
    total += blocks[index]->block_length();
  }

  lastblock = index;

#ifdef DEBUG_OUTPUT
  cerr << "firstblock = " << firstblock << "   ms1 = " << ms1 << endl;
  cerr << "lastblock =  " << lastblock  << "   ms2 = " << ms2 << endl;
#endif

  // delete tail
  while (lastblock < blocks.size() - 1) {
    remove_block(lastblock + 1);
  }

  // calculate new end
  int new_end_offset = static_cast<int>(static_cast<double>(blocks[lastblock]->get_frequency()) * (to_ms - ms2) / 1e3);

  WAVblock *a = dynamic_cast<WAVblock *>(blocks[lastblock]);
  MP3block *b = dynamic_cast<MP3block *>(blocks[lastblock]);

  //
  if (a && static_cast<double>(blocks[lastblock]->get_end_sample_offset() - new_end_offset) / blocks[lastblock]->get_frequency() * 1e3 < 1.0) {
    cerr << "less than 1ms" << endl;
    new_end_offset = blocks[lastblock]->num_samples();
  }
  else
  if (b && (b->samples_per_frame() + 50) > blocks[lastblock]->get_end_sample_offset() - new_end_offset) {
    cerr << "less than 1 frame  " << blocks[lastblock]->get_end_sample_offset() - new_end_offset << endl;
    new_end_offset = blocks[lastblock]->num_samples();
  }

  // calculate new start
  int new_start_offset = static_cast<int>(static_cast<double>(blocks[firstblock]->get_frequency()) * (from_ms - ms1) / 1e3);

#ifdef DEBUG_OUTPUT
  cerr << "new_start_offset = " << new_start_offset << "     new_end_offset = " << new_end_offset << endl;
#endif

  blocks[lastblock]->set_end_sample_offset(new_end_offset + blocks[lastblock]->get_begin_sample_offset());
  blocks[firstblock]->set_begin_sample_offset(new_start_offset + blocks[firstblock]->get_begin_sample_offset());

  // delete head
  while (firstblock > 0) {
    remove_block(0);
    firstblock--;
  }

}

int BlockList::select_block(int offset_ms) {
  if (blocks.size() == 0) return 0;

  int totalms = total_ms();
  int total = blocks[0]->block_length();
  unsigned int index = 0;
  unsigned int numblocks = blocks.size();
  int ms = 0;
  unsigned int selected_block;

  while (total < offset_ms && index < numblocks) {
    ms = total;
    index++;
    total += blocks[index]->block_length();
  }

  selected_block = index;

  index = 0;

  if (selected_block < numblocks)   // selected block is not last, erase tail of list
    blocks.erase(blocks.begin() + selected_block + 1, blocks.end());

  if (selected_block > 0)   // selected block is not first, erase head of list
    blocks.erase(blocks.begin(), blocks.begin() + selected_block);

#ifdef DEBUG_OUTPUT
  cerr << "select_block: number of blocks = " << blocks.size() << endl;
#endif

  return ms;
}


void BlockList::delete_block(int offset_ms) {
  if (blocks.size() == 0 || playing) return;

  int index = index_at_offset(offset_ms);

  SoundBlock *sb = block_at(index);
  delete sb;
  blocks.erase(blocks.begin() + index);

  chg = true;
  block_list_changed.emit();
}

void BlockList::resize_block(int offset_ms) {
  SoundBlock *sb = block_at_offset(offset_ms);

  if (sb == 0) return;

  DialogResize *dresize = manage(new DialogResize(sb));
//  dresize->destroy.connect(slot(this, &BlockList::resize_destroyed));
}

int BlockList::index_at_offset(int offset_ms) {
  if (blocks.size() == 0) return -1;

  int total = blocks[0]->block_length();
  unsigned int index = 0;
  unsigned int numblocks = blocks.size();

  while (total < offset_ms && index < numblocks) {
    index++;
    total += blocks[index]->block_length();
  }

  return static_cast<int>(index);
}

SoundBlock *BlockList::block_at_offset(int offset_ms) {
  int index = index_at_offset(offset_ms);

  SoundBlock *sb = block_at(index);
  return sb;
}

void BlockList::sound_block_changed() {
  chg = true;

  if (!cutting) block_list_changed.emit();      // if cutting do not emit signal!

#ifdef DEBUG_OUTPUT
  cerr << "sound_block_changed" << endl;
#endif
}

void BlockList::cut(int from_ms, int to_ms) {
  int totalms = total_ms();

  if (from_ms > totalms || from_ms == to_ms || playing) return;
  if (to_ms > totalms) to_ms = totalms;

	cutting = true;
	
  int total = blocks[0]->block_length();
  unsigned int index = 0;
  unsigned int numblocks = blocks.size();
  int ms1 = 0, ms2;
  unsigned int firstblock, lastblock;

  while (total < from_ms && index < numblocks) {
    ms1 = total;
    index++;
    total += blocks[index]->block_length();
  }

  firstblock = index;
  ms2 = ms1;

  while (total < to_ms && index < numblocks - 1) {
    ms2 = total;
    index++;
    total += blocks[index]->block_length();
  }

  lastblock = index;

#ifdef DEBUG_OUTPUT
  cerr << "firstblock = " << firstblock << "   ms1 = " << ms1 << endl;
  cerr << "lastblock =  " << lastblock  << "   ms2 = " << ms2 << endl;
#endif


  if (firstblock == lastblock) {      // if cutting in one sound block then break it

#ifdef DEBUG_OUTPUT
    cerr << "cutting in one sound block" << endl;
#endif

    // use RTTI do get sound block class
    MP3block *b1 = dynamic_cast<MP3block *>(block_at(firstblock));
    WAVblock *b2 = dynamic_cast<WAVblock *>(block_at(firstblock));

    // break soundblock to 2 parts
    if (b1) {     // MP3block
       MP3block *copy = new MP3block(*b1);

       copy->block_done.connect(slot(this, &BlockList::block_play_done));
//       copy->playedMiliSec.connect(slot(this, &BlockList::blockPlayedMiliSecs));
       copy->block_changed.connect(slot(this, &BlockList::sound_block_changed));
       copy->export_done.connect(slot(this, &BlockList::block_export_done));

       blocks.insert(blocks.begin() + firstblock, copy);
    }
    else if (b2) {   // WAVblock
       WAVblock *copy = new WAVblock(*b2);

       copy->block_done.connect(slot(this, &BlockList::block_play_done));
//       copy->playedMiliSec.connect(slot(this, &BlockList::blockPlayedMiliSecs));
       copy->block_changed.connect(slot(this, &BlockList::sound_block_changed));
       copy->export_done.connect(slot(this, &BlockList::block_export_done));

       blocks.insert(blocks.begin() + firstblock, copy);
    }

    lastblock++;     // set lastblock to new created copy
  }

  // set new begin and end
  int new_end_offset = static_cast<int>(static_cast<double>(blocks[firstblock]->get_frequency()) * (from_ms - ms1) / 1e3);
  int new_start_offset = static_cast<int>(static_cast<double>(blocks[lastblock]->get_frequency()) * (to_ms - ms2) / 1e3);

//  WAVblock *a = dynamic_cast<WAVblock *>(blocks[lastblock]);
  MP3block *b = dynamic_cast<MP3block *>(blocks[lastblock]);

//  WAVblock *c = dynamic_cast<WAVblock *>(blocks[firstblock]);
  MP3block *a = dynamic_cast<MP3block *>(blocks[firstblock]);

#ifdef DEBUG_OUTPUT
  cerr << "blocklist last : new_start_offset: " << new_start_offset << "  num:" << blocks[lastblock]->num_samples() << endl;
  cerr << "blocklist first: new_end_offset:   " << new_end_offset   << "  num:" << blocks[firstblock]->num_samples() << endl;
#endif

  blocks[firstblock]->set_end_sample_offset(new_end_offset + blocks[firstblock]->get_begin_sample_offset());
  blocks[lastblock]->set_begin_sample_offset(new_start_offset + blocks[lastblock]->get_begin_sample_offset());

  // delete blocks between first and last block
  int remove = lastblock - firstblock;
  while (remove > 1) {
    remove_block(firstblock + 1);
    remove--;
  }

#ifdef DEBUG_OUTPUT
  cerr << "[firstblock+1]->block_length()  " << blocks[firstblock+1]->block_length() << endl;
#endif



  if (b && b->samples_per_frame() == b->num_samples()) {
    remove_block(firstblock + 1);
  }

  if (a && a->samples_per_frame() == a->num_samples()){
    remove_block(firstblock);
  }

  // if truncated block is very small (< 1 ms) then delete it (because parameters are
  // are passed in ms the reminder can be max. 1 ms)
  if (blocks[firstblock + 1]->block_length() == 0) remove_block(firstblock + 1);
  if (blocks[firstblock]->block_length() == 0) remove_block(firstblock);

  cutting = false;
  block_list_changed.emit();      // emit signal
}


void BlockList::paste(int offset_ms, vector<SoundBlock *> &b) {
  int totalms = total_ms();

  if (offset_ms > totalms || playing) return;

  int total = blocks[0]->block_length();
  unsigned int index = 0;
  unsigned int numblocks = blocks.size();
  int ms1 = 0;
  unsigned int firstblock;

  while (total < offset_ms && index < numblocks) {
    ms1 = total;
    index++;
    total += blocks[index]->block_length();
  }

  firstblock = index;

  // use RTTI do get sound block class
  MP3block *b1 = dynamic_cast<MP3block *>(block_at(firstblock));
  WAVblock *b2 = dynamic_cast<WAVblock *>(block_at(firstblock));

  // break soundblock to 2 parts
  if (b1) {     // MP3block
     MP3block *copy = new MP3block(*b1);

     copy->block_done.connect(slot(this, &BlockList::block_play_done));
//     copy->playedMiliSec.connect(slot(this, &BlockList::blockPlayedMiliSecs));
     copy->block_changed.connect(slot(this, &BlockList::sound_block_changed));
     copy->export_done.connect(slot(this, &BlockList::block_export_done));

     blocks.insert(blocks.begin() + firstblock, copy);
  }
  else if (b2) {   // WAVblock
     WAVblock *copy = new WAVblock(*b2);

     copy->block_done.connect(slot(this, &BlockList::block_play_done));
//     copy->playedMiliSec.connect(slot(this, &BlockList::blockPlayedMiliSecs));
     copy->block_changed.connect(slot(this, &BlockList::sound_block_changed));
     copy->export_done.connect(slot(this, &BlockList::block_export_done));

     blocks.insert(blocks.begin() + firstblock, copy);
  }


  // set new begin and end
  int new_end_offset = static_cast<int>(static_cast<double>(blocks[firstblock]->get_frequency()) * (offset_ms - ms1) / 1e3);
  int new_start_offset = static_cast<int>(static_cast<double>(blocks[firstblock + 1]->get_frequency()) * (offset_ms - ms1) / 1e3);

  blocks[firstblock]->set_end_sample_offset(new_end_offset + blocks[firstblock]->get_begin_sample_offset());
  blocks[firstblock + 1]->set_begin_sample_offset(new_start_offset + blocks[firstblock + 1]->get_begin_sample_offset());

  // connect signals
  index = 0;
  while (index < b.size()) {
    b[index]->block_done.connect(slot(this, &BlockList::block_play_done));
//    b[index]->playedMiliSec.connect(slot(this, &BlockList::blockPlayedMiliSecs));
    b[index]->block_changed.connect(slot(this, &BlockList::sound_block_changed));
    b[index]->export_done.connect(slot(this, &BlockList::block_export_done));

    index++;
  }

  blocks.insert(blocks.begin() + firstblock + 1, b.begin(), b.end());
  block_list_changed.emit();      // emit signal
}

void BlockList::paste_before(int offset_ms, vector<SoundBlock *> &b) {
  // connect signals to this block
  unsigned int index = 0;
  while (index < b.size()) {
    b[index]->block_done.connect(slot(this, &BlockList::block_play_done));
//    b[index]->playedMiliSec.connect(slot(this, &BlockList::blockPlayedMiliSecs));
    b[index]->block_changed.connect(slot(this, &BlockList::sound_block_changed));
    b[index]->export_done.connect(slot(this, &BlockList::block_export_done));

    index++;
  }

  blocks.insert(blocks.begin() + index_at_offset(offset_ms), b.begin(), b.end());
  block_list_changed.emit();      // emit signal
}

void BlockList::paste_after(int offset_ms, vector<SoundBlock *> &b) {
  // connect signals to this block
  unsigned int index = 0;
  while (index < b.size()) {
    b[index]->block_done.connect(slot(this, &BlockList::block_play_done));
//    b[index]->playedMiliSec.connect(slot(this, &BlockList::blockPlayedMiliSecs));
    b[index]->block_changed.connect(slot(this, &BlockList::sound_block_changed));
    b[index]->export_done.connect(slot(this, &BlockList::block_export_done));

    index++;
  }

  blocks.insert(blocks.begin() + index_at_offset(offset_ms) + 1, b.begin(), b.end());
  block_list_changed.emit();      // emit signal
}


vector<SoundBlock *> BlockList::get_vector_copy() {
	unsigned int index = 0;
	MP3block *mp3b, *newmp3;
	WAVblock *wavb, *newwav;
	vector<SoundBlock *> result;
	
	// create copies of sound blocks
	// use RTTI do get type of sound block
	while (index < blocks.size()) {
	  mp3b = dynamic_cast<MP3block*>(blocks[index]);
	
	  if (mp3b) {                             // is it MP3block?
	    newmp3 = new MP3block(*mp3b);
	    result.push_back(newmp3);
	  }
	  else {
  	  wavb = dynamic_cast<WAVblock*>(blocks[index]);  // is it WAVblock?
   	  if (wavb) {
  	    newwav = new WAVblock(*wavb);
  	    result.push_back(newwav);
   	  }
   	  else
   	    cerr << "Unknown class of SoundBlock???" << endl;
	  }
	  index++;
	}

	return result;
}

bool BlockList::export_list_WAV(string file, int ch, int f, int bps, DialogProgress *dp,
                                string decoder_path, string conversion_path) {

#ifdef DEBUG_OUTPUT
    cerr << "decoder:   " << decoder_path << endl;
    cerr << "converter: " << conversion_path << endl;
#endif

  export_type = EXPORT_WAV;

  if (playing || blocks.size() == 0) {
    export_done.emit();
    return false;
  }

  export_progress = dp;
  export_mpg123_path = decoder_path;
  export_sox_path = conversion_path;

  // check whether output file exists
  ifstream is;
  is.open(file.c_str(), ios::binary | ios::in);

  // file exists, ask to rewrite it
  if (is.is_open()) {
    string question;
    question = _("File ") + file + _(" already exists.\nRewrite file?");
		Gnome::Dialog *q = Gnome::Dialogs::ok_cancel_modal(question);
		is.close();
		
		if (q->run_and_close() == 1) {
      export_done.emit();
		  return false;    // don't rewrite file
		}
  }

  // open output stream
  export_stream.open(file.c_str(), ios::binary | ios::out);			// open output stream

  if (!export_stream.is_open()) {
    cerr << "can not open output file!!" << endl;
    export_done.emit();
    return false;
  }

  // write empty WAV header
  Utils::write_WAV_header(export_stream, ch, f, bps);


  // set actual block to first block
  actualIndex = 0;

  export_channels = ch;
  export_freq = f;
  export_bps = bps;

  // export first block
  actual = blocks[actualIndex];
  actual->export_WAV(export_stream, export_channels, export_freq, export_bps, export_progress,
                    export_mpg123_path, export_sox_path);

  return true;
}


bool BlockList::export_list_MP3(string file, string encoder_name, string encoder_path, int bitrate, int frequency,
		   MP3block::ChannelMode channel_mode, bool crc, bool priv, bool original, bool copyright,
		   vector<string> VBR_options, vector<string> encoder_options, bool always_process, bool strict_equal,
		   DialogTagEditor *id3, DialogProgress *dp) {

  export_type = EXPORT_MP3;

  if (playing || blocks.size() == 0) {
    export_done.emit();
    return false;
  }

  // check whether output file exists
  ifstream is;
  is.open(file.c_str(), ios::binary | ios::in);

  // file exists, ask to rewrite it
  if (is.is_open()) {
    string question;
    question = _("File ") + file + _(" already exists.\nRewrite file?");
		Gnome::Dialog *q = Gnome::Dialogs::ok_cancel_modal(question);
		is.close();
		
		if (q->run_and_close() == 1) {
      export_done.emit();
		  return false;    // don't rewrite file
		}
  }

  // open output stream
  export_stream.open(file.c_str(), ios::binary | ios::out);			// open output stream

  if (!export_stream.is_open()) {
    cerr << "can not open output file!!" << endl;
    export_done.emit();
    return false;
  }
		
  // set actual block to first block
  actualIndex = 0;

  export_progress = dp;

  export_freq = frequency;
	export_encoder_name = encoder_name;
	export_bitrate = bitrate;
	export_channel_mode = channel_mode;
  export_crc = crc;
  export_priv = priv;
  export_original = original;
  export_copyright = copyright;
  export_VBR_options = VBR_options;
  export_encoder_options = encoder_options;
  export_strict_equal = strict_equal;
  export_always_process = always_process;
  export_id3 = id3;
  export_encoder_path = encoder_path;

  // export first block
  actual = blocks[actualIndex];
  actual->export_MP3(export_stream, export_encoder_name, export_encoder_path, export_bitrate, export_freq, export_channel_mode,
                    export_crc, export_priv, export_original, export_copyright, export_VBR_options,
                    export_encoder_options, export_always_process, export_strict_equal, export_progress);

  return true;
}


void BlockList::block_export_done() {
  actualIndex++;

  if (!export_progress->cancel_state() && actualIndex < static_cast<signed int>(blocks.size())) {
    actual = blocks[actualIndex];

    switch(export_type) {
      case(EXPORT_WAV) : actual->export_WAV(export_stream, export_channels, export_freq, export_bps,
                                           export_progress, export_mpg123_path, export_sox_path); break;

      case(EXPORT_MP3) : actual->export_MP3(export_stream, export_encoder_name, export_encoder_path, export_bitrate, export_freq,
                    export_channel_mode, export_crc, export_priv, export_original, export_copyright, export_VBR_options,
                    export_encoder_options, export_always_process, export_strict_equal, export_progress);
                    break;
    }
  }
  else {

    // write lengths to WAV header
    switch(export_type) {
      case(EXPORT_WAV) : Utils::write_WAV_header_len(export_stream); break;
      case(EXPORT_MP3) : Utils::write_ID3_tag(export_stream, export_id3); break;
    }

    export_stream.close();

#ifdef DEBUG_OUTPUT
    cerr << "Export done" << endl;
#endif

    export_done.emit();
  }
}
