/***************************************************************************
             dialogexportmp3.h  -  Dialog for setting MP3 parameters
            ---------------------------------------------------------
    begin                : Fri Mar 30 2001
    copyright            : (C) 2001 by Ladislav Slezk
    email                : slezak@penguin.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DIALOGEXPORTMP3_H
#define DIALOGEXPORTMP3_H

#include <gtk--/window.h>
#include <gtk--/frame.h>
#include <gtk--/box.h>
#include <gtk--/label.h>
#include <gtk--/button.h>
#include <gtk--/buttonbox.h>
#include <gtk--/optionmenu.h>
#include <gtk--/menu.h>
#include <gtk--/checkbutton.h>
#include <gtk--/table.h>

#include <gnome--/stock.h>
#include <libgnome/gnome-i18n.h>
#include <gnome--/file-entry.h>

#include <sigc++/signal_system.h>
#include <string>

#include "mp3block.h"
#include "dialogtageditor.h"
#include "dialogvbroptions.h"
#include "dialoglameoptions.h"

/**Dialog for exporting into MP3 file
  *@author Ladislav Slezk
  */

class DialogExportMP3 : public Gtk::Window {

  public:

	  DialogExportMP3();
   	~DialogExportMP3();
   	
		SigC::Signal1<void, bool> selection_done;    // signal - OK or Cancel pressed

		int get_frequency() {return freq;}
		int get_bitrate() {return bitrate;}
    string get_filename();
    bool get_private() {return stream_private->get_active();}
    bool get_crc() {return stream_crc->get_active();}
    bool get_original() {return stream_original->get_active();}
    bool get_copyright() {return stream_copyright->get_active();}
    MP3block::ChannelMode get_channel_mode() {return mode;}
    DialogTagEditor *get_tag_editor();
    vector<string> get_vbr_options();
    string get_encoder_name() {return encoder_name;}
    vector<string> get_encoder_options();
   	
  private:

    Gnome::FileEntry *fentry;
    Gtk::CheckButton *tag_checkbutton;
    Gtk::Button *tag_button;
    Gtk::Button *stream_bitrate_button;

    Gtk::CheckButton *stream_private;
    Gtk::CheckButton *stream_crc;
    Gtk::CheckButton *stream_original;
    Gtk::CheckButton *stream_copyright;

    Gtk::MenuItem *br8, *br16, *br24, *br32, *br40, *br48, *br56, *br64, *br80, *br96,
                  *br112, *br128, *br144, *br160, *br192, *br224, *br256, *br320, *br_vbr;

    Gtk::MenuItem *f16000, *f22050, *f24000, *f32000, *f44100, *f48000;

    DialogTagEditor *tag_editor;
    DialogVBRoptions *vbr_options;
    DialogLameOptions *lame_options;

    int freq, bitrate;
    MP3block::ChannelMode mode;
    string encoder_name;

    void show_tag_editor();
    void tag_editor_destroyed();
    void show_vbr_options();
    void vbr_dialog_destroyed();
    void show_lame_options();
    void lame_dialog_destroyed();

    void encoder_changed(string name);

    void state_edit_tag();
    void freq_changed(int f);
    void bitrate_changed(int br);
    void mode_changed(MP3block::ChannelMode m);
    void button_pressed(bool ok);

    int delete_event_impl(GdkEventAny* e);

};

#endif
