/***************************************************************************
          dialogproject.cpp  -  display dialog with list of blocks
         ----------------------------------------------------------
    begin                : Fri Sep 8 2000
    copyright            : (C) 2000 by Ladislav Slezk
    email                : slezak@penguin.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "dialogproject.h"
#include "soundblock.h"
#include "utils.h"

#include <gnome--/app.h>
#include <libgnome/gnome-i18n.h>

#include <strstream>
#include <string>

#include <gtk--/buttonbox.h>

DialogProject::DialogProject(BlockList *bl) : Gtk::Window(GTK_WINDOW_DIALOG) {
  blist = bl;
	selected_item = -1;    // nothing selected
	
	vector<string> head;
	head.push_back(_("No."));
	head.push_back(_("File"));
	head.push_back(_("Type"));
	head.push_back(_("Channels"));
	head.push_back(_("Frequency"));
	head.push_back(_("Info"));
	head.push_back(_("Length"));
	
	clist = manage(new Gtk::CList(head));
	
	clist->set_column_auto_resize(0, true);
	clist->set_column_auto_resize(1, true);
	clist->set_column_auto_resize(2, true);
	clist->set_column_auto_resize(3, true);
	clist->set_column_auto_resize(4, true);
	clist->set_column_auto_resize(5, true);
	clist->set_column_auto_resize(6, true);

	refresh();	
	
	clist->select_row.connect(slot(this, &DialogProject::selected));
	
  swindow = manage(new Gtk::ScrolledWindow());
  swindow->set_policy(GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  swindow->add(*clist);

//	bup = manage(new Gtk::Button(_("Move up")));
  bup = manage(new Gnome::StockButton(GNOME_STOCK_BUTTON_UP));
  bup->clicked.connect(slot(this, &DialogProject::bup_pressed));

//	bdown = manage(new Gtk::Button(_("Move down")));
  bdown = manage(new Gnome::StockButton(GNOME_STOCK_BUTTON_DOWN));
  bdown->clicked.connect(slot(this, &DialogProject::bdown_pressed));
	
  bdelete = manage(new Gtk::Button(_("Delete block")));
  bdelete->clicked.connect(slot(this, &DialogProject::bdelete_pressed));

  bresize = manage(new Gtk::Button(_("Resize block ...")));
  bresize->clicked.connect(slot(this, &DialogProject::bresized_pressed));

//  bclose = manage(new Gtk::Button(_("Close")));
  bclose = manage(new Gnome::StockButton(GNOME_STOCK_BUTTON_CLOSE));
  bclose->clicked.connect(hide.slot());

  Gtk::VBox *boxB = manage(new Gtk::VBox(false, 5));

  boxV = manage(new Gtk::VButtonBox(GTK_BUTTONBOX_START));
  boxV->pack_start(*bup, false, false, 0);
  boxV->pack_start(*bdown, false, false, 0);
  boxV->pack_start(*bdelete, false, false, 0);
  boxV->pack_start(*bresize, false, false, 0);

  boxB->pack_start(*boxV, false, false, 0);
  boxB->pack_end(*bclose, false, false, 0);

  boxH = manage(new Gtk::HBox(false, 5));
  boxH->pack_start(*swindow, true, true, 0);
  boxH->pack_start(*boxB, false, false, 0);

  add(*boxH);
	set_modal(true);
  set_border_width(5);
  set_default_size(450, 300);
  set_title(_("List of blocks"));

  bclose->set_flags(GTK_CAN_DEFAULT);
  bclose->grab_default();

  show_all();
}

DialogProject::~DialogProject() {

}


void DialogProject::refresh() {
	int no = 0;
	clist->clear();
	while (no < blist->num_blocks()) {
		SoundBlock *b = blist->block_at(no);
		strstream ss1, ss2, ss3;
		vector<string> line;
		string s;
		
		ss1 << no;
		ss1 >> s;
		line.push_back(s);
		line.push_back(b->get_file_name());
		line.push_back(b->get_type());
		ss2 << b->get_channels();
		ss2 >> s;
		line.push_back(s);
		ss3 << b->get_frequency();
		ss3 >> s;
		line.push_back(s + " Hz");
		line.push_back(b->block_info());
	
  	line.push_back(Utils::ms2str(b->block_length()));
		
		clist->rows().push_back(line);
		no++;
	}
}

void DialogProject::selected(int y, int x, GdkEvent *) {
  selected_item = y;

#ifdef DEBUG_OUTPUT
  cerr << "Selected " <<  y << endl;
#endif
}

void DialogProject::bdelete_pressed() {
  if (selected_item == -1) return;

  blist->remove_block(selected_item);
  selected_item = -1;
  refresh();
  clist->row(selected_item).select();
}

void DialogProject::bresized_pressed() {
  if (selected_item == -1) return;
  dresize = manage(new DialogResize(blist->block_at(selected_item)));
  dresize->destroy.connect(slot(this, &DialogProject::refresh));
}

void DialogProject::bup_pressed() {
  if (selected_item == -1) return;

  if (blist->block_to_begin(selected_item)) {
    selected_item--;
    refresh();
    clist->row(selected_item).select();
  }
}

void DialogProject::bdown_pressed() {
  if (selected_item == -1) return;

  if (blist->block_to_end(selected_item)) {
    selected_item++;
    refresh();
    clist->row(selected_item).select();
  }
}

