/***************************************************************************
                     mainwin.cpp  -  main application window
                    -----------------------------------------
    begin                : Fri Jul 7 2000
    copyright            : (C) 2000 by Ladislav Slezk
    email                : slezak@penguin.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "mainwin.h"
#include "utils.h"

#include <libgnome/gnome-i18n.h>
#include <libgnome/gnome-util.h>    // gnome_pixmap_file

#include <gdk--/color.h>
#include <gnome--/dialog.h>
#include <gnome--/stock.h>
#include <gtk--/separator.h>

#include <strstream.h>
#include <iostream.h>

#include "../pixmaps/quickedit_logo.xpm"
#include "../pixmaps/quickedit_logo_mini.xpm"
#include "../pixmaps/quickedit_pause.xpm"
#include "../pixmaps/quickedit_pause_mini.xpm"
#include "../pixmaps/quickedit_play.xpm"
#include "../pixmaps/quickedit_play_mini.xpm"
#include "../pixmaps/quickedit_stop.xpm"
#include "../pixmaps/quickedit_stop_mini.xpm"
#include "../pixmaps/quickedit_zoom_fit.xpm"
#include "../pixmaps/quickedit_zoom_fit_mini.xpm"
#include "../pixmaps/quickedit_zoom_in.xpm"
#include "../pixmaps/quickedit_zoom_in_mini.xpm"
#include "../pixmaps/quickedit_zoom_out.xpm"
#include "../pixmaps/quickedit_zoom_out_mini.xpm"

MainWin::MainWin() : Gnome::App("Quickedit", "Quickedit") {

  set_policy(false, true, false);
  set_default_size(450, 350);
  dproject = 0;

  Gtk::HBox *hb = manage(new Gtk::HBox());

  // create status bar labels
  l1 = manage(new Gtk::Label(_("Ready"), 0.0));
  Gtk::Label *l2 = manage(new Gtk::Label(_("Pos:"), 0.0));
  l3 = manage(new Gtk::Label(Utils::ms2str(0), 0.0));
  Gtk::Label *l4 = manage(new Gtk::Label(_("Time:"), 0.0));
  l5 = manage(new Gtk::Label(Utils::ms2str(0), 0.0));
  Gtk::Label *l6 = manage(new Gtk::Label(_("Total:"), 0.0));
  l7 = manage(new Gtk::Label(Utils::ms2str(0), 0.0));
  l8 = manage(new Gtk::Label);

  // put labels to box
  hb->pack_start(*l1, false, false, 5);
  hb->pack_start(*l2, false, false, 5);
  hb->pack_start(*l3, false, false, 3);
  hb->pack_start(*l4, false, false, 5);
  hb->pack_start(*l5, false, false, 3);
  hb->pack_start(*l6, false, false, 5);
  hb->pack_start(*l7, false, false, 3);
  hb->pack_end(*l8, false, false, 5);   // selection

  set_statusbar(*hb);   // put box to status bar

  hruler1 = manage(new Gtk::HRuler());
  view = manage(new View());
  view->reconfigured.connect(slot(this, &MainWin::refresh));

  view->set_usize(800,10);

  // connect motion notify events
  view->motion_notify_event.connect(hruler1->motion_notify_event.slot());
  view->motion_notify_event.connect(SigC::slot(this, &MainWin::set_position));

  // connect mouse button events
  view->button_press_event.connect(SigC::slot(this, &MainWin::mouse_button_pressed));
  view->button_release_event.connect(SigC::slot(this, &MainWin::mouse_button_released));

  hruler1->motion_notify_event.connect(SigC::slot(this, &MainWin::set_position));

  vbox1 = manage(new Gtk::VBox(false, 0));
  scrolledwindow1 = manage(new Gtk::ScrolledWindow());
  scrolledwindow1->set_policy(GTK_POLICY_AUTOMATIC, GTK_POLICY_NEVER);

  hruler1->set_metric(GTK_PIXELS);
  hruler1->set_range(0, 800, 0, 100);

  // put ruler and view into box
  vbox1->pack_start(*hruler1, false, true, 0);
  vbox1->pack_start(*view);

  // put box to scrolled window
  scrolledwindow1->add_with_viewport(*vbox1);

  // set main window contents
  set_contents(*scrolledwindow1);


  // create main menu
  vector<Gnome::UI::Info> menu, file_menu, edit_menu, project_menu, view_menu, help_menu, export_menu;

  Gnome::UI::Item export_as_WAV(N_("Export as WAV file ..."), slot(this, &MainWin::export_WAV));
  export_menu.push_back(export_as_WAV);
  Gnome::UI::Item export_as_MP3(N_("Export as MP3 file ..."), slot(this, &MainWin::export_MP3));
  export_menu.push_back(export_as_MP3);

  Gnome::UI::Item add_block(N_("Add new block ..."), slot(this, &MainWin::add_block));
	add_block.set_accel('A');
  file_menu.push_back(add_block);
  file_menu.push_back(Gnome::UI::Separator());
  file_menu.push_back(Gnome::MenuItems::Open(slot(this, &MainWin::open_project)));
  file_menu.push_back(Gnome::MenuItems::Save(slot(this, &MainWin::save_project)));
  file_menu.push_back(Gnome::MenuItems::SaveAs(slot(this, &MainWin::save_project_as)));
  Gnome::UI::SubTree export_as(N_("Export"), export_menu);
  file_menu.push_back(export_as);
  file_menu.push_back(Gnome::UI::Separator());
  file_menu.push_back(Gnome::MenuItems::Exit(slot(this, &MainWin::exit_cb)));

  edit_menu.push_back(Gnome::MenuItems::Cut(slot(this, &MainWin::cut_from_project)));
  edit_menu.push_back(Gnome::MenuItems::Copy(slot(this, &MainWin::copy_to_clipboard)));
  edit_menu.push_back(Gnome::MenuItems::Paste(slot(this, &MainWin::paste_from_clipboard)));
  edit_menu.push_back(Gnome::UI::Separator());
  edit_menu.push_back(Gnome::MenuItems::SelectAll(slot(this, &MainWin::select_all)));
  edit_menu.push_back(Gnome::UI::Separator());
  edit_menu.push_back(Gnome::MenuItems::Preferences(slot(this, &MainWin::show_config_dialog)));

  Gnome::UI::Item list_block(N_("List of blocks ..."), slot(this, &MainWin::display_project));
	list_block.set_accel('L');
  project_menu.push_back(list_block);
  project_menu.push_back(Gnome::UI::Separator());

  Gnome::UI::Item play_block(Gnome::UI::IconXpm(const_cast<const char **>(quickedit_play_mini_xpm)), N_("Play project/selection"), slot(this, &MainWin::play_project));
	play_block.set_accel('Z');
  project_menu.push_back(play_block);
  Gnome::UI::Item pause_block(Gnome::UI::IconXpm(const_cast<const char **>(quickedit_pause_mini_xpm)), N_("Pause playing"), slot(this, &MainWin::pause_project));
	pause_block.set_accel('X');
  project_menu.push_back(pause_block);
  Gnome::UI::Item stop_block(Gnome::UI::IconXpm(const_cast<const char **>(quickedit_stop_mini_xpm)), N_("Stop playing"), slot(this, &MainWin::stop_project));
	stop_block.set_accel('C');
  project_menu.push_back(stop_block);

  help_menu.push_back(Gnome::MenuItems::About(slot(this, &MainWin::about_cb)));

  Gnome::UI::Item zoom_in_item(Gnome::UI::IconXpm(const_cast<const char **>(quickedit_zoom_in_mini_xpm)), N_("Zoom in"), slot(this, &MainWin::zoom_in));
	zoom_in_item.set_accel('+');
  Gnome::UI::Item zoom_out_item(Gnome::UI::IconXpm(const_cast<const char **>(quickedit_zoom_out_mini_xpm)),N_("Zoom out"), slot(this, &MainWin::zoom_out));
	zoom_out_item.set_accel('-');
  Gnome::UI::Item zoom_fit_item(Gnome::UI::IconXpm(const_cast<const char **>(quickedit_zoom_fit_mini_xpm)),N_("Fit in window"), slot(this, &MainWin::zoom_fit));
	zoom_fit_item.set_accel('=');
  Gnome::UI::Item refresh_view(N_("Refresh view"), slot(this, &MainWin::refresh));
  view_menu.push_back(zoom_in_item);
  view_menu.push_back(zoom_out_item);
  view_menu.push_back(zoom_fit_item);
  view_menu.push_back(Gnome::UI::Separator());
  view_menu.push_back(refresh_view);

  menu.push_back(Gnome::Menus::File(file_menu));
  menu.push_back(Gnome::Menus::Edit(edit_menu));
  menu.push_back(Gnome::UI::Menu(_("P_roject"), project_menu));
  menu.push_back(Gnome::UI::Menu(_("_View"), view_menu));
  menu.push_back(Gnome::Menus::Help(help_menu));

  create_menus(menu);

  // create toolbar
  vector<Gnome::UI::Info> toolbar;

  toolbar.push_back(Gnome::UI::Item(Gnome::UI::Icon(GNOME_STOCK_PIXMAP_OPEN),
    N_("Open"), slot(this, &MainWin::open_project), N_("Open project")));

  toolbar.push_back(Gnome::UI::Item(Gnome::UI::Icon(GNOME_STOCK_PIXMAP_ADD),
    N_("Add"), slot(this, &MainWin::add_block), N_("Add file to project")));

  toolbar.push_back(Gnome::UI::Item(Gnome::UI::Icon(GNOME_STOCK_PIXMAP_SAVE),
    N_("Save"), slot(this, &MainWin::save_project), N_("Save project")));

  toolbar.push_back(Gnome::UI::Separator());

  toolbar.push_back(Gnome::UI::Item(Gnome::UI::IconXpm(const_cast<const char **>(quickedit_play_xpm)),
    N_("Play"), slot(this, &MainWin::play_project), N_("Play project/selection")));

  toolbar.push_back(Gnome::UI::Item(Gnome::UI::IconXpm(const_cast<const char **>(quickedit_pause_xpm)),
    N_("Pause"), slot(this, &MainWin::pause_project), N_("Pause playing")));

  toolbar.push_back(Gnome::UI::Item(Gnome::UI::IconXpm(const_cast<const char **>(quickedit_stop_xpm)),
    N_("Stop"), slot(this, &MainWin::stop_project), N_("Stop playing")));

  toolbar.push_back(Gnome::UI::Separator());

  toolbar.push_back(Gnome::UI::Item(Gnome::UI::IconXpm(const_cast<const char **>(quickedit_zoom_in_xpm)),
    N_("Zoom in"), slot(this, &MainWin::zoom_in), N_("Zoom in")));

  toolbar.push_back(Gnome::UI::Item(Gnome::UI::IconXpm(const_cast<const char **>(quickedit_zoom_out_xpm)),
    N_("Zoom out"), slot(this, &MainWin::zoom_out), N_("Zoom out")));

  toolbar.push_back(Gnome::UI::Item(Gnome::UI::IconXpm(const_cast<const char **>(quickedit_zoom_fit_xpm)),
    N_("To window"), slot(this, &MainWin::zoom_fit), N_("Fit to window")));
			
  create_toolbar(toolbar);

  // connect project signals
	project.play_done.connect(slot(this, &MainWin::status_ready));
  project.block_list_changed.connect(slot(this, &MainWin::project_changed));
  project.export_done.connect(slot(this, &MainWin::export_completed));
  project.play_block_done.connect(slot(this, &MainWin::block_played));

  /****** Create popup menu ******/
  // create sub menu Selection->Export
  Gtk::Menu *popup_menu_selection_export = manage(new Gtk::Menu);
  // insert items to selection submenu
  popup_menu_selection_export_WAV = manage(new Gtk::MenuItem(_("Export as WAV ...")));
  popup_menu_selection_export_WAV->activate.connect(slot(this, &MainWin::export_selection_WAV));
  popup_menu_selection_export_MP3 = manage(new Gtk::MenuItem(_("Export as MP3 ...")));
  popup_menu_selection_export_MP3->activate.connect(slot(this, &MainWin::export_selection_MP3));
  popup_menu_selection_export->append(*popup_menu_selection_export_WAV);
  popup_menu_selection_export->append(*popup_menu_selection_export_MP3);
  popup_menu_selection_export->show_all();

  // create popup submenu 'Selection'
  popup_menu_selection = manage(new Gtk::Menu);
  // insert items to selection submenu
  popup_menu_selection_play_start = manage(new Gtk::MenuItem(_("Play start")));
  popup_menu_selection_play_start->activate.connect(slot(this, &MainWin::play_start_selection));
  popup_menu_selection->append(*popup_menu_selection_play_start);
  popup_menu_selection_play_end = manage(new Gtk::MenuItem(_("Play end")));
  popup_menu_selection_play_end->activate.connect(slot(this, &MainWin::play_end_selection));
  popup_menu_selection->append(*popup_menu_selection_play_end);
  popup_menu_selection->append(*manage(new Gtk::MenuItem));       // append separator
//  popup_menu_selection_change_start = manage(new Gtk::MenuItem(_("Change start")));
//  popup_menu_selection_change_start->activate.connect(slot(this, &MainWin::nothing_cb));
//  popup_menu_selection->append(*popup_menu_selection_change_start);
//  popup_menu_selection_change_end = manage(new Gtk::MenuItem(_("Change end")));
//  popup_menu_selection_change_end->activate.connect(slot(this, &MainWin::nothing_cb));
//  popup_menu_selection->append(*popup_menu_selection_change_end);
//  popup_menu_selection->append(*manage(new Gtk::MenuItem));       // append separator
  popup_menu_selection_export_submenu = manage(new Gtk::MenuItem(_("Export")));
  popup_menu_selection_export_submenu->set_submenu(*popup_menu_selection_export);
  popup_menu_selection->append(*popup_menu_selection_export_submenu);
  popup_menu_selection->show_all();

  // nothing selected - deactivate selection items in popup menu
  set_active_popup_selection(false);


  // create popup submenu 'Block'
  Gtk::Menu *popup_menu_block = manage(new Gtk::Menu);
  // insert items to block submenu
  popup_menu_block_select = manage(new Gtk::MenuItem(_("Select")));
  popup_menu_block_select->activate.connect(slot(this, &MainWin::select_this_block));
  popup_menu_block->append(*popup_menu_block_select);
  popup_menu_block_resize = manage(new Gtk::MenuItem(_("Resize ...")));
  popup_menu_block_resize->activate.connect(slot(this, &MainWin::resize_this_block));
  popup_menu_block->append(*popup_menu_block_resize);
  popup_menu_block_delete = manage(new Gtk::MenuItem(_("Delete")));
  popup_menu_block_delete->activate.connect(slot(this, &MainWin::delete_this_block));
  popup_menu_block->append(*popup_menu_block_delete);
  popup_menu_block->show_all();

  // create popup submenu 'Paste'
  popup_menu_paste = manage(new Gtk::Menu);
  // insert items to block submenu
  popup_menu_paste_before = manage(new Gtk::MenuItem(_("Paste before block")));
  popup_menu_paste_before->activate.connect(slot(this, &MainWin::paste_before));
  popup_menu_paste_after = manage(new Gtk::MenuItem(_("Paste after block")));
  popup_menu_paste_after->activate.connect(slot(this, &MainWin::paste_after));
  popup_menu_paste->append(*popup_menu_paste_before);
  popup_menu_paste->append(*popup_menu_paste_after);
  popup_menu_paste->show_all();

  // clipboard is empty - deactivate paste items in popup menu
  set_active_popup_paste(false);

  // create main popup menu
  popup_menu = manage(new Gtk::Menu);
  Gtk::Menu_Helpers::MenuList &popup_menu_list = popup_menu->items();
  // insert items to main popup menu
  popup_menu_list.push_back(Gtk::Menu_Helpers::MenuElem(_("This block"), *popup_menu_block));
  popup_menu_list.push_back(Gtk::Menu_Helpers::MenuElem(_("Selection"), *popup_menu_selection));
  popup_menu_list.push_back(Gtk::Menu_Helpers::MenuElem(_("Clipboard"), *popup_menu_paste));

  // set menu accelerators to main window
  popup_menu_selection->accelerate(*this);
	
  // initialize class members
  fs = 0;
  fss = 0;
  select = false;
  start_selection = -1;
  end_selection = -1;
  selection = 0;
  clipboard = 0;
  close_canceled = false;
  project_totalMS = 0;
  exit_after_save = false;
	dx_wav = 0;
	dx_mp3 = 0;
	export_progress = 0;
	export_selection = false;
	dialog_options = 0;

	show_all();                   // realize all widgets
	
	read_config();	              // read configuration
	zoom_fit();              // set view to fit to the main window
}


int MainWin::timeout() {  // redisplay position in status bar
  int new_pos;

  if (project.is_playing()) {
//  	l5->set(Utils::ms2str(project.playedMS()));
//    cerr << "OSS bytes = " << ossdev.get_processed_bytes() << "   OSS ms = " << ossdev.get_processed_ms() << endl;

    int proc_ms = ossdev.get_processed_ms() + played_ms;
  	l5->set(Utils::ms2str(proc_ms));
  	
  	new_pos = static_cast<int>(static_cast<double>(proc_ms) / project_totalMS * view->width());
  	
//  	new_pos = static_cast<int>(static_cast<double>(project.playedMS()) / project_totalMS * view->width());
  	if (new_pos == last_pos) return 1;
  	
   	view->draw_pos(new_pos);
   	last_pos = new_pos;
  }
  else
    if (selection && selection->is_playing()) {
      int proc_ms = ossdev.get_processed_ms() + played_ms + start_selection;

//    	l5->set(Utils::ms2str(start_selection + selection->playedMS()));
    	l5->set(Utils::ms2str(proc_ms));

//    	new_pos = static_cast<int>(static_cast<double>(selection->playedMS()) / project_totalMS * view->width())
    	new_pos = static_cast<int>(static_cast<double>(proc_ms) / project_totalMS * view->width());
//              	+ static_cast<int>(static_cast<double>(start_selection) / project_totalMS * view->width());
    	if (new_pos == last_pos) return 1;
    	
     	view->draw_pos(new_pos);
     	last_pos = new_pos;
    }

  return 1;
}


MainWin::~MainWin(){
  Gtk::Main::quit();
}


void MainWin::close() {
  // stop if playing
  if (project.is_playing()) {
    project.stop_list();
    timer_connection.disconnect();
  }

  if (selection && selection->is_playing()) {          // is selection playing?
    selection->stop_list();
    timer_connection.disconnect();
  }

  ask_save_changes();       // ask if save changes

#ifdef DEBUG_OUTPUT
  cerr << "close_canceled = " << close_canceled << endl;
#endif
}

gint MainWin::delete_event_impl(GdkEventAny* e) {
  close_canceled = false;
  close();

  return close_canceled || exit_after_save;
}

void MainWin::nothing_cb() {
		Gnome::Dialog *q = Gnome::Dialogs::warning(*this, "Not implemented yet ...");
		q->set_modal(true);
		q->run_and_close();
}


void MainWin::exit_cb() {
  close_canceled = false;
  close();

  if (!close_canceled && !exit_after_save) {
    write_config();			// write configuration data
    destroy();
  }
}

void MainWin::about_cb() {  // display about dialog

  // set authors
  vector<string> author;
  author.push_back("Ladislav Slezk <slezak@penguin.cz>");

  // set pixmap name
  char *l = gnome_pixmap_file("quickedit_logo.png");
  string logo;

  if (l != NULL) logo = l;

#ifdef DEBUG_OUTPUT
  cerr << "logo: " << logo << endl;
#endif

  about = new Gnome::About("Quickedit", VERSION, "(C) 2000 Ladislav Slezk",
    author, _("Simple sound editor for Gnome environment.\n \n"
    "This program is free software; you can redistribute it and/or modify "
    "it under the terms of the GNU General Public License as published by "
    "the Free Software Foundation; either version 2 of the License,\n"
    "or (at your option) any later version"), logo);

  about->set_parent(*this);     // set parent main window
  about->run_and_close();       // show dialog and wait for close

  delete about;                 // delete dialog
}

int MainWin::set_position(GdkEventMotion *evnt) {
  int totms = project_totalMS;
  int w = view->width();
  int mis = static_cast<int>(static_cast<double>(totms) * evnt->x / w);

  if (mis < 0) mis = 0;
  if (mis > totms) mis = totms;

  l3->set(Utils::ms2str(mis));    // display position of cursor in status bar
  cursor_position_ms = mis;

  if (select && start_selection != mis) {
    int start_pix = static_cast<int>(static_cast<double>(start_selection) / totms * w);
    int end_pix = static_cast<int>(evnt->x);

    if (end_pix < 0) end_pix = 0;
    if (end_pix > w) end_pix = w;

    view->draw_selection(start_pix, static_cast<int>(evnt->x));

    // set temporary start and end of selection
    int tmpstart = start_selection;
    if (mis < tmpstart) {
      int tmp = mis;
      mis = tmpstart;
      tmpstart = tmp;
    }

    // diplay selection properties in status bar
    statusbar_display_selection(tmpstart, mis);
  }

  return 1;   // event is handled
}

void MainWin::add_block() {
  if (fs == 0) {
    fs = manage(new Gtk::FileSelection(_("Add new sound block")));
    fs->set_modal(true);
    fs->complete("*.wav");
    fs->get_ok_button()->clicked.connect(bind(slot(this, &MainWin::file_selected), true));
    fs->get_cancel_button()->clicked.connect(bind(slot(this, &MainWin::file_selected), false));
  }

//  preview buttons
//  Gtk::HButtonBox *bb = manage(new Gtk::HButtonBox);
//  fs->get_action_area()->pack_end(*bb, true, true, 0);
//
//  Gtk::Button *b1 = manage(new Gtk::Button("Test"));
//  bb->pack_start(*b1, false, false, 0);
//  Gtk::Button *b2 = manage(new Gtk::Button("Button"));
//  bb->pack_start(*b2, false, false, 0);

  fs->show_all();   // display dialog
}

void MainWin::save_project_as() {
  if (fss == 0) {
    fss = manage(new Gtk::FileSelection(_("Save project as")));
    fss->set_modal(true);
    fss->complete("*.qed");
    fss->get_ok_button()->clicked.connect(bind(slot(this, &MainWin::save_file_selected), true));
    fss->get_cancel_button()->clicked.connect(bind(slot(this, &MainWin::save_file_selected), false));
  }

  fss->show_all();   // display dialog
}

void MainWin::file_selected(bool ok) {
  fs->hide();

  if (!ok) return;

	string fn = fs->get_filename();

#ifdef DEBUG_OUTPUT
  cerr << fn << "    ok: " << ok << endl;
#endif

  // get extension
  int pos = fn.rfind('.');
  if (pos > 0) {
    string ext(fn, pos, fn.size());
#ifdef DEBUG_OUTPUT
    cerr << "Extension:  " << ext << endl;
#endif

		for (unsigned int i = 0; i < ext.length(); i++)
			ext[i] = tolower(ext[i]);
			
    if (ext == ".wav") {
      WAVblock *wavb = new WAVblock;      // create new WAV block

      if (!wavb->assign_file(fn)) {        // assign file
        delete wavb;                      // if assign failed delete block
        return;
      }
      project.add_block(wavb);             // add block to project
      wavb->scan_file(0, wavb->num_samples());              // scan file
#ifdef DEBUG_OUTPUT
      cerr << "Samples: " << wavb->num_samples() << endl;
#endif

			refresh();
    }
    else
    if (ext == ".mp3") {
      MP3block *mpegb = new MP3block;   // create new MP3 block

      if (!mpegb->assign_file(fn)) {     // assign file
      	delete mpegb;                      // if assign failed delete block
        return;
      }
      project.add_block(mpegb);
      mpegb->scan_file(0, mpegb->num_samples());
      mpegb->scan_done.connect(slot(this, &MainWin::refresh));
#ifdef DEBUG_OUTPUT
      cerr << "Samples: " << mpegb->num_samples() << endl;
#endif

			refresh();
    }
  }
}

void MainWin::save_file_selected(bool ok) {
  fss->hide();

  if (!ok) return;

	string fn = fss->get_filename();
	
	if (project.save_BlockList_to_file(fn)) {
	  set_window_title(fn);                 // set window title to project file name
  	refresh();
  }

  if (exit_after_save) destroy();
}

void MainWin::play_project() {
  if (project.is_playing() || (selection && selection->is_playing())) return;            // already playing

  l1->set(_("Playing"));
  timer_connection = Gtk::Main::timeout.connect(SigC::slot(this, &MainWin::timeout), 40);
  last_pos = -1;
  played_ms = 0;

  if (selection) {                  // play selection or project
    selection->play_list(&ossdev, option_mpg123_path);
  }
  else {
    project.play_list(&ossdev, option_mpg123_path);
  }
}

void MainWin::stop_project() {
  timer_connection.disconnect();

  if (project.is_playing())
    project.stop_list();
  else
    if (selection && selection->is_playing())
       selection->stop_list();
}

void MainWin::pause_project() {
	if (project.is_paused()) {
  	project.unpause_list();
  	l1->set(_("Playing"));
	}
 	else {
 	  if (project.is_playing()) {
    	project.pause_list();
  	  l1->set(_("Paused"));
	  }
	}
	
	if (selection  && selection->is_paused()) {
  	selection->unpause_list();
  	l1->set(_("Playing"));
	}
 	else {
 	  if (selection && selection->is_playing()) {
    	selection->pause_list();
  	  l1->set(_("Paused"));
	  }
	}
}


void MainWin::refresh() {
  project_totalMS = project.total_ms();
  // redisplay
	view->clear();
  project.display_blocks(view->get_pixmap(), view->get_GC());
	view->refresh();
	
	// redraw playing position
	if (project.is_playing()) view->draw_pos(last_pos);
	
	// redraw selection
	if (start_selection >= 0 && end_selection >= 0) {
	  int start_pix = static_cast<int>(static_cast<double>(start_selection) / project_totalMS * view->width());
	  int end_pix = static_cast<int>(static_cast<double>(end_selection) / project_totalMS * view->width());
	
	  view->draw_selection(start_pix, end_pix);
	}

  // set new total time
  hruler1->set_range(0, project_totalMS/1000.0, 0, 0);
  string s = Utils::ms2str(project_totalMS);
  l7->set(s);
}

void MainWin::open_project() {
  if (fso == 0) {
    fso = manage(new Gtk::FileSelection(_("Open project")));
    fso->set_modal(true);
    fso->complete("*.qed");
    fso->get_ok_button()->clicked.connect(bind(slot(this, &MainWin::open_file_selected), true));
    fso->get_cancel_button()->clicked.connect(bind(slot(this, &MainWin::open_file_selected), false));
  }

  fso->show();
}

void MainWin::open_file_selected(bool ok) {
  fso->hide();

  if (!ok) return;

	string fn = fso->get_filename();

	if (project.load_BlockList_from_file(fn))
	  set_window_title(fn);           // set window title to project file name
  else
	  set_title("Quickedit");         // open failed - set window title to Quickedit
	
  if (selection) {
    if (selection->is_playing()) selection->stop_list();
    delete selection;
    selection = 0;
    end_selection = -1;
    start_selection = -1;
  }

	// refresh project dialog if exists
  if (dproject != 0) dproject->refresh();
	
  project_totalMS = project.total_ms();
	refresh();
}

void MainWin::display_project() {
  if (dproject == 0)
    dproject = manage(new DialogProject(&project));
  else
    dproject->show_all();

}

void MainWin::status_ready() {
  // delete playing position
  view->clear_pos();

  // set status bar texts
  l1->set(_("Ready"));
  l5->set(Utils::ms2str(0));
  played_ms = 0;

  timer_connection.disconnect();
}

void MainWin::zoom_in() {
  int curr_width = view->width();

  view->set_usize(static_cast<gint>(curr_width * 1.5), 10);

#ifdef DEBUG_OUTPUT
  cerr << "Width: " << view->width() << endl;
#endif
}

void MainWin::zoom_out() {
  int curr_width = view->width();

  if (static_cast<gint>(curr_width / 1.5) < width() - 8)
    zoom_fit();
  else
    view->set_usize(static_cast<gint>(curr_width / 1.5), 10);

#ifdef DEBUG_OUTPUT
  cerr << "Width: " << view->width() << endl;
#endif
}

void MainWin::zoom_fit() {
  view->set_usize(width() - 8, 10);

#ifdef DEBUG_OUTPUT
  cerr << "Width: " << view->width() << endl;
#endif
}

gint MainWin::mouse_button_pressed(GdkEventButton *eb) {
  if (eb->button == 1) {                                // test left button
    if (selection && selection->is_playing()) return 1;  // selection is playing -- don't allow new selection

    select = true;       // selection started
    start_selection = static_cast<int>(eb->x * project_totalMS / view->width());

    // diplay selection properties in status bar
    statusbar_display_selection(start_selection);
  }

  if (eb->button == 3) {                                // test right button
    if (project.is_playing() || (selection && selection->is_playing())) return 1;   // playing - don't display popup menu

    popup_menu->popup(eb->button, eb->time);            // display popup menu
    cursor_position_ms = static_cast<int>(eb->x * project_totalMS / view->width());
  }

  return 1;     // event is handled
}

gint MainWin::mouse_button_released(GdkEventButton *eb) {

  if (eb->button == 1 && select) {
    select = false;
    end_selection = static_cast<int>(eb->x * project_totalMS / view->width());

    // check boundaries
    if (end_selection < 0) end_selection = 0;
    if (end_selection > project_totalMS) end_selection = project_totalMS;

    if (start_selection > end_selection) {        // start of selection must be lower than end of selection
      int tmp = end_selection;

      end_selection = start_selection;
      start_selection = tmp;
    }

    if (end_selection == start_selection) {       // nothing selected
      end_selection = -1;
      start_selection = -1;
      view->clear_selection();
    }

#ifdef DEBUG_OUTPUT
    cerr << "MainWin::mouse_button_released: from: " << start_selection << "  to: "<< end_selection << endl;
#endif

    if (selection) {                  // delete old selection (which must not be playing)
      delete selection;
      selection = 0;
    }

    if (start_selection >= 0 && end_selection >= 0) {           // create new selection
      selection = new BlockList(project);                       // create new copy
      selection->select(start_selection, end_selection);        // set selected part
      selection->play_done.connect(slot(this, &MainWin::status_ready));
      selection->export_done.connect(slot(this, &MainWin::export_completed));
      selection->play_block_done.connect(slot(this, &MainWin::block_played));

      // diplay selection properties in status bar
      statusbar_display_selection(start_selection, end_selection);
      set_active_popup_selection(true);

    }
    else {
      statusbar_display_selection(-1);    // delete selection info from status bar
      set_active_popup_selection(false);
    }
  }

  // support wheel mouse if enabled
  if (option_use_wheel && eb->button == 4) zoom_in();
  if (option_use_wheel && eb->button == 5) zoom_out();

  return 1;     // event is handled
}

void MainWin::select_all() {
  if (selection && selection->is_playing()) return;          // is selection playing?

  if (selection) delete selection;                          // delete old selection

  selection = new BlockList(project);                       // create new copy
  selection->play_done.connect(slot(this, &MainWin::status_ready));
  selection->export_done.connect(slot(this, &MainWin::export_completed));
  selection->play_block_done.connect(slot(this, &MainWin::block_played));

  start_selection = 0;                                      // start = 0
  end_selection = selection->total_ms();                     // end = total length

  refresh();    // display selection
}

void MainWin::project_changed() {
#ifdef DEBUG_OUTPUT
  cerr << "project_changed" << endl;
#endif

  project_totalMS = project.total_ms();

  // project changed: delete selection, add ' *' to title
  if (selection && selection->is_playing()) return;          // is selection playing?
  if (selection) delete selection;                          // delete old selection
  selection = 0;

  // no selection
  start_selection = -1;
  end_selection = -1;

  // delete text in status bar
  statusbar_display_selection(-1);

  string title = project.get_file_name();
  if (title.empty()) title = _("Untitled");  // project has no title

  // set window title to project file name, display modified mark
  set_window_title(title, true);

 	// refresh project dialog if exists
  if (dproject != 0) dproject->refresh();

  refresh();    // refresh display
}

void MainWin::save_project() {
  string fn = project.get_file_name();

	if (fn.size() > 0) {                             // test file name
	  if (project.save_BlockList_to_file(fn, false))    // save project
      set_window_title(fn);                        // save succesfull -- refresh window title
  }
	else
	  save_project_as();                        // project has never been saved
}

void MainWin::ask_save_changes() {
  // if project was changed then ask to save it
  if (project.is_changed()) {
    // create modal question dialog
    Gnome::Dialog *qm = Gnome::Dialogs::question_modal(*this, _("Project was changed!\nDo you wish to save changes?"));

    // add button to dialog
    qm->append_button(_("Cancel"), GNOME_STOCK_PIXMAP_CLOSE);

    // connect callbacks
    qm->connect(0, bind(slot(this, &MainWin::save_changes_answer), 0));   // button 'Yes'
    qm->connect(1, bind(slot(this, &MainWin::save_changes_answer), 1));   // button 'No'
    qm->connect(2, bind(slot(this, &MainWin::save_changes_answer), 2));   // button 'Cancel'

    // set default button 'Yes'
    qm->set_default(0);

    // show dialog and wait for button press
    qm->show_all();
		qm->run_and_close();
  }
}

void MainWin::save_changes_answer(int butt) {
  switch (butt) {
    case (0) : close_canceled = false;
      exit_after_save = true; save_project(); break;   // button 'Yes'    -- save project and exit
    case (1) : close_canceled = false; break;               // button 'No'     -- exit
    case (2) : close_canceled = true;  break;               // button 'Cancel' -- don't exit

    default: break;                                         // other button???
  }
}

void MainWin::set_window_title(string title, bool modified = false) {
  string end_string = (modified) ? "] *" : "]";                   // add '*' if modified
  string new_title = (title.empty()) ? _("Untitled") : title;     // set title to 'Untiled' if title is empty

  set_title("Quickedit  - [" + new_title + end_string);
}

void MainWin::select_this_block() {
  if (selection && selection->is_playing()) return;   // old selection is playing, no change

  if (selection) {                  // delete old selection
    delete selection;
    selection = 0;
  }

  selection = new BlockList(project);                       // create new copy
  selection->play_done.connect(slot(this, &MainWin::status_ready));
  selection->export_done.connect(slot(this, &MainWin::export_completed));
  selection->play_block_done.connect(slot(this, &MainWin::block_played));

  start_selection = selection->select_block(cursor_position_ms);    // select one block
  end_selection = start_selection + selection->total_ms();

  if (start_selection != end_selection) {
    // diplay selection properties in status bar
    statusbar_display_selection(start_selection, end_selection);
    refresh();    // display selection on screen
    set_active_popup_selection(true);
  }
  else {          // nothing selected
    statusbar_display_selection(-1);
    start_selection = -1;
    end_selection = -1;

    delete selection;   // delete created selection
    set_active_popup_selection(false);
    selection = 0;
  }
}

void MainWin::statusbar_display_selection(int start_ms, int end_ms = -1){
  string text;

  if (start_ms >= 0) {
    text = _("  Selection:  Start: ") + Utils::ms2str(start_ms);
    if (end_ms >= 0)
      text += _("  End: ") + Utils::ms2str(end_ms) + _("  Length: ") + Utils::ms2str(end_ms - start_ms);
  }
  else
    text = "";

  l8->set(text);
}

void MainWin::delete_this_block() {
  if (selection && selection->is_playing()) return;   // old selection is playing, no change
  if (project.is_playing()) return;

  project.delete_block(cursor_position_ms);
}

void MainWin::resize_this_block() {
  if (selection && selection->is_playing()) return;   // old selection is playing, no change
  if (project.is_playing()) return;

  project.resize_block(cursor_position_ms);
}

void MainWin::copy_to_clipboard() {
  if (selection == 0) return;         // nothing selected

  if (clipboard) delete clipboard;    // delete old clipboard

  clipboard = new BlockList(*selection);        // create new clipboard
  set_active_popup_paste(true);                 // enable popup menu paste items

#ifdef DEBUG_OUTPUT
  cerr << "Clipboard: copy created" << endl;
#endif
}

void MainWin::cut_from_project() {
  if (selection == 0) return;         // nothing selected

  copy_to_clipboard();                // copy selection to clipboard

#ifdef DEBUG_OUTPUT
  cerr << "Clipboard: cut from: " << start_selection << "  to: "<< end_selection << endl;
#endif

  project.cut(start_selection, end_selection);      // cut from project

  statusbar_display_selection(-1);    // delete selection info from status bar
  start_selection = -1;
  end_selection = -1;

  delete selection;
  selection = 0;

  refresh();              // refresh display
}

void MainWin::paste_before() {
  if (clipboard == 0) return;

#ifdef DEBUG_OUTPUT
  cerr << "Clipboard paste before:  pos_ms: " << cursor_position_ms << endl;
#endif

  vector<SoundBlock *> part = clipboard->get_vector_copy();
  project.paste_before(cursor_position_ms, part);
}

void MainWin::paste_after() {
  if (clipboard == 0) return;

#ifdef DEBUG_OUTPUT
  cerr << "Clipboard paste after:  pos_ms: " << cursor_position_ms << endl;
#endif

  vector<SoundBlock *> part = clipboard->get_vector_copy();
  project.paste_after(cursor_position_ms, part);
}

void MainWin::paste_from_clipboard() {
  if (clipboard == 0) return;

#ifdef DEBUG_OUTPUT
  cerr << "Clipboard paste:  pos_ms: " << cursor_position_ms << endl;
#endif

  vector<SoundBlock *> part = clipboard->get_vector_copy();
  project.paste(cursor_position_ms, part);
}

void MainWin::export_WAV() {
	if (dx_wav == 0) {
	  dx_wav = manage (new DialogExportWAV);
	  dx_wav->export_ack.connect(slot(this, &MainWin::export_WAV_selected));
	}
	else
	  dx_wav->show();
	
  dx_wav->set_frequency(option_exportwav_freq);
  dx_wav->set_bps(option_exportwav_bps);
  dx_wav->set_channels(option_exportwav_ch);
}

void MainWin::export_MP3() {
  if (dx_mp3 == 0) {
    dx_mp3 = manage(new DialogExportMP3);
    dx_mp3->selection_done.connect(slot(this, &MainWin::export_MP3_selected));
  }
  else
    dx_mp3->show();
}

void MainWin::export_selection_WAV() {
  export_selection = true;
  export_WAV();
}

void MainWin::export_selection_MP3() {
  export_selection = true;
  export_MP3();
}


void MainWin::export_WAV_selected(bool ok) {
#ifdef DEBUG_OUTPUT
  cerr << "MainWin: Export WAV ok:  " << ok << endl;

  if (ok) {
    cerr << "bps  = " << dx_wav->get_bps() << endl;
    cerr << "ch   = " << dx_wav->get_channels() << endl;
    cerr << "freq = " << dx_wav->get_frequency() << endl;

    cerr << "File = " << dx_wav->get_filename() << endl;
  }
#endif

  if (ok && !dx_wav->get_filename().empty()) {
    l1->set(_("Exporting"));
    export_progress = manage (new DialogProgress(_("Please wait ..."), _("Exporting ...")));

    // actualize configuration
    option_exportwav_bps = dx_wav->get_bps();
    option_exportwav_ch = dx_wav->get_channels();
    option_exportwav_freq = dx_wav->get_frequency();

    // export selection or whole project
    if (export_selection && selection != 0)
      selection->export_list_WAV(dx_wav->get_filename(), dx_wav->get_channels(), dx_wav->get_frequency(),
        dx_wav->get_bps(), export_progress, option_mpg123_path, option_sox_path);
    else
      project.export_list_WAV(dx_wav->get_filename(), dx_wav->get_channels(), dx_wav->get_frequency(),
        dx_wav->get_bps(), export_progress, option_mpg123_path, option_sox_path);
  }

}

void MainWin::export_MP3_selected(bool ok) {

  if (ok && !dx_mp3->get_filename().empty()) {
    l1->set(_("Exporting"));
    export_progress = manage (new DialogProgress(_("Please wait ..."), _("Exporting ...")));

    // export selection or whole project
    if (export_selection && selection != 0)
      selection->export_list_MP3(dx_mp3->get_filename(), dx_mp3->get_encoder_name(), option_lame_path, dx_mp3->get_bitrate(),
        dx_mp3->get_frequency(), dx_mp3->get_channel_mode(), dx_mp3->get_crc(), dx_mp3->get_private(),
        dx_mp3->get_original(), dx_mp3->get_copyright(), dx_mp3->get_vbr_options(), dx_mp3->get_encoder_options(),
        option_mp3export_process, option_mp3export_strictequal, dx_mp3->get_tag_editor(), export_progress);
    else
      project.export_list_MP3(dx_mp3->get_filename(), dx_mp3->get_encoder_name(), option_lame_path, dx_mp3->get_bitrate(),
        dx_mp3->get_frequency(), dx_mp3->get_channel_mode(), dx_mp3->get_crc(), dx_mp3->get_private(),
        dx_mp3->get_original(), dx_mp3->get_copyright(), dx_mp3->get_vbr_options(), dx_mp3->get_encoder_options(),
        option_mp3export_process, option_mp3export_strictequal, dx_mp3->get_tag_editor(), export_progress);
  }

}

void MainWin::export_completed() {
  l1->set(_("Ready"));               // set text in status bar
  export_selection = false;

  if (export_progress != 0) {
    export_progress->destroy();      // destroy export progress dialog
    export_progress = 0;
  }
}

void MainWin::set_active_popup_selection(bool active) {
  popup_menu_selection_export_WAV->set_sensitive(active);
  popup_menu_selection_export_MP3->set_sensitive(active);
  popup_menu_selection_play_start->set_sensitive(active);
  popup_menu_selection_play_end->set_sensitive(active);
//  popup_menu_selection_change_start->set_sensitive(active);
//  popup_menu_selection_change_end->set_sensitive(active);
  popup_menu_selection_export_submenu->set_sensitive(active);
}

void MainWin::set_active_popup_paste(bool active) {
  popup_menu_paste_before->set_sensitive(active);
  popup_menu_paste_after->set_sensitive(active);
}

void MainWin::show_config_dialog() {
	if (dialog_options == 0) {
	  dialog_options = manage (new DialogOptions);
	  dialog_options->ok_pressed.connect(slot(this, &MainWin::get_config_from_dialog));
	}
	else
	  dialog_options->show();
	
  dialog_options->set_foreground_color(option_fg_color);
  dialog_options->set_background_color(option_bg_color);
  dialog_options->set_lame_path(option_lame_path);
  dialog_options->set_mpg123_path(option_mpg123_path);
  dialog_options->set_sox_path(option_sox_path);
  dialog_options->set_use_wheel(option_use_wheel);
  dialog_options->set_oss_device(option_oss_device);

  strstream ss;
  ss << option_end_time;
  string end_time_str;
  ss >> end_time_str;
  dialog_options->set_end_time(end_time_str);

  dialog_options->set_mp3_always_process(option_mp3export_process);
  dialog_options->set_mp3_strict_equal(option_mp3export_strictequal);

}

void MainWin::read_config() {
  // read foreground color from config file - default is green
  string fg_color_string = config.get_string(string("/quickedit/Colors/foreground=green"));

  // read background color from config file - default is black
  string bg_color_string = config.get_string(string("/quickedit/Colors/background=black"));

  // parse color from string
  option_fg_color.parse(fg_color_string);
  option_bg_color.parse(bg_color_string);

  // set colors
  view->set_foreground_color(option_fg_color);
  view->set_background_color(option_bg_color);

  option_end_time = config.get_integer(string("/quickedit/Selection/end time=3000"));

  // get program paths
  option_mpg123_path = config.get_string(string("/quickedit/Programs/mpg123=mpg123"));
  option_lame_path = config.get_string(string("/quickedit/Programs/lame=lame"));
  option_sox_path = config.get_string(string("/quickedit/Programs/sox=sox"));

	// get OSS device path
  option_oss_device = config.get_string(string("/quickedit/OSS/device=/dev/dsp"));
  ossdev.set_device(option_oss_device);

  // get wheel support
  option_use_wheel = config.get_bool(string("/quickedit/Mouse/use_wheel=false"));

  // get wav export options
  option_exportwav_bps = config.get_integer(string("/quickedit/WAV export/bits_per_sample=16"));
  option_exportwav_ch = config.get_integer(string("/quickedit/WAV export/channels=2"));
  option_exportwav_freq = config.get_integer(string("/quickedit/WAV export/frequency=44100"));

  // get MP3 export parameters
  option_mp3export_process = config.get_bool(string("/quickedit/MP3 export/always_process=true"));
  option_mp3export_strictequal = config.get_bool(string("/quickedit/MP3 export/strict_equal=true"));

#ifdef DEBUG_OUTPUT
  cerr << "Quickedit settings: " << endl;
  cerr << "  foreground color : " << fg_color_string << endl;
  cerr << "  background color : " << bg_color_string << endl;
  cerr << "  end time         : " << option_end_time << endl;
  cerr << "  mpg123 path      : " << option_mpg123_path << endl;
  cerr << "  lame path        : " << option_lame_path << endl;
  cerr << "  sox path         : " << option_sox_path << endl;
  cerr << "  use wheel        : " << option_use_wheel << endl;
  cerr << "  OSS device       : " << option_oss_device << endl;

  cerr << "  WAV bps          : " << option_exportwav_bps << endl;
  cerr << "  WAV freq         : " << option_exportwav_freq << endl;
  cerr << "  WAV channels     : " << option_exportwav_ch << endl;
  cerr << endl;
#endif

}

void MainWin::get_config_from_dialog() {
  if (dialog_options == 0) return;

  option_fg_color = dialog_options->get_foreground_color();
  option_bg_color = dialog_options->get_background_color();

  view->set_foreground_color(option_fg_color);
  view->set_background_color(option_bg_color);

  option_lame_path = dialog_options->get_lame_path();
  option_mpg123_path = dialog_options->get_mpg123_path();
  option_sox_path = dialog_options->get_sox_path();
  option_use_wheel = dialog_options->get_use_wheel();

  option_oss_device = dialog_options->get_oss_device();
  ossdev.set_device(option_oss_device);

  string tmp = dialog_options->get_end_time();
  strstream tmps;
  tmps << tmp;
  tmps >> option_end_time;

  option_mp3export_process = dialog_options->get_mp3_always_process();
  option_mp3export_strictequal = dialog_options->get_mp3_strict_equal();


#ifdef DEBUG_OUTPUT
  cerr << "  end time         : " << option_end_time << endl;
#endif
}

void MainWin::write_config() {
  // write program paths
  config.set_string(string("/quickedit/Programs/mpg123"), option_mpg123_path);
  config.set_string(string("/quickedit/Programs/lame"), option_lame_path);
  config.set_string(string("/quickedit/Programs/sox"), option_sox_path);

  // write wheel support
  config.set_bool(string("/quickedit/Mouse/use_wheel"), option_use_wheel);

  // write end time
  config.set_integer(string("/quickedit/Selection/end time"), option_end_time);

  // write colors
  strstream fgstream;
  fgstream << '#';
  fgstream.fill('0');
  fgstream.width(2);
  fgstream.setf(ios::hex, ios::basefield);
  fgstream << (option_fg_color.get_red() >> 8);
  fgstream.fill('0');
  fgstream.width(2);
  fgstream.setf(ios::hex, ios::basefield);
  fgstream << (option_fg_color.get_green() >> 8);
  fgstream.fill('0');
  fgstream.width(2);
  fgstream.setf(ios::hex, ios::basefield);
  fgstream << (option_fg_color.get_blue() >> 8);

  string fg_string;
  fgstream >> fg_string;


  strstream bgstream;
  bgstream << '#';
  bgstream.fill('0');
  bgstream.width(2);
  bgstream.setf(ios::hex, ios::basefield);
  bgstream << (option_bg_color.get_red() >> 8);
  bgstream.fill('0');
  bgstream.width(2);
  bgstream.setf(ios::hex, ios::basefield);
  bgstream << (option_bg_color.get_green() >> 8);
  bgstream.fill('0');
  bgstream.width(2);
  bgstream.setf(ios::hex, ios::basefield);
  bgstream << (option_bg_color.get_blue() >> 8);

  string bg_string;
  bgstream >> bg_string;


  config.set_string(string("/quickedit/Colors/foreground"), fg_string);
  config.set_string(string("/quickedit/Colors/background"), bg_string);

#ifdef DEBUG_OUTPUT
  cerr << "Foreground color : " << fg_string << endl;
  cerr << "Background color : " << bg_string << endl;
#endif

  // write OSS device file
  config.set_string(string("/quickedit/OSS/device"), option_oss_device);

  // write WAV export parameters
  config.set_integer(string("/quickedit/WAV export/bits_per_sample"), option_exportwav_bps);
  config.set_integer(string("/quickedit/WAV export/channels"), option_exportwav_ch);
  config.set_integer(string("/quickedit/WAV export/frequency"), option_exportwav_freq);

  // write MP3 export parameters
  config.set_bool(string("/quickedit/MP3 export/always_process"), option_mp3export_process);
  config.set_bool(string("/quickedit/MP3 export/strict_equal"), option_mp3export_strictequal);

}

void MainWin::play_end_selection() {
  if (project.is_playing() || (selection && selection->is_playing())) return;            // already playing

  l1->set(_("Playing"));
  timer_connection = Gtk::Main::timeout.connect(SigC::slot(this, &MainWin::timeout), 40);
  last_pos = -1;

  selection->play_list(&ossdev, option_mpg123_path, selection->total_ms() - option_end_time);
}

void MainWin::play_start_selection() {
  if (project.is_playing() || (selection && selection->is_playing())) return;            // already playing

  l1->set(_("Playing"));
  timer_connection = Gtk::Main::timeout.connect(SigC::slot(this, &MainWin::timeout), 40);
  last_pos = -1;

  selection->play_list(&ossdev, option_mpg123_path, -1, option_end_time);
}

void MainWin::block_played(int ms) {
  played_ms += ms;
}
