/***************************************************************************
                     mainwin.h  -  main application window
                    ---------------------------------------
    begin                : Fri Jul 7 2000
    copyright            : (C) 2000 by Ladislav Slezk
    email                : slezak@penguin.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MAINWIN_H
#define MAINWIN_H

#include <vector>
#include <gtk--/label.h>
#include <gtk--/button.h>
#include <gtk--/frame.h>
#include <gtk--/main.h>

#include <gtk--/menu.h>

#include <gtk--/window.h>
#include <gtk--/ruler.h>
#include <gtk--/pixmap.h>
#include <gtk--/box.h>
#include <gtk--/viewport.h>
#include <gtk--/scrolledwindow.h>
#include <gtk--/fileselection.h>

#include <gnome--/app.h>
#include <gnome--/appbar.h>
#include <gnome--/about.h>

#include "view.h"
#include "blocklist.h"
#include "wavblock.h"
#include "mp3block.h"
#include "ossdevice.h"
#include "dialogproject.h"
#include "dialogexportwav.h"
#include "dialogexportmp3.h"
#include "dialogprogress.h"
#include "dialogoptions.h"
#include "gnomeconfig.h"

#include "subprocess.h"

using namespace std;

class MainWin : public Gnome::App {

  public:
    MainWin(); 
    ~MainWin();


  protected:

    void close();

    // Callbacks
    virtual gint delete_event_impl(GdkEventAny *e);

    void nothing_cb();

    void about_cb();
    void exit_cb();

    void add_block();
    void play_project();
    void stop_project();
    void pause_project();
    void file_selected(bool);
    void save_project();
    void save_project_as();
    void save_file_selected(bool);
    void open_project();
    void open_file_selected(bool);
    void display_project();
    void status_ready();
    void zoom_in();
    void zoom_out();
    void zoom_fit();
    void select_all();
    void select_this_block();
    void delete_this_block();
    void resize_this_block();
		
    gint mouse_button_pressed(GdkEventButton*);
    gint mouse_button_released(GdkEventButton*);

    int timeout();
    void refresh();
    void project_changed();
    void ask_save_changes();
    void save_changes_answer(int butt);
    void set_window_title(string title, bool modified = false);
    void statusbar_display_selection(int start_ms, int end_ms = -1);

    void copy_to_clipboard();
    void cut_from_project();
    void paste_before();
    void paste_after();
    void paste_from_clipboard();

    void play_end_selection();
    void play_start_selection();

    void export_WAV();
    void export_WAV_selected(bool ok);
    void export_selection_WAV();
    void export_selection_MP3();
    void export_MP3();
    void export_MP3_selected(bool ok);
    void export_completed();

    void set_active_popup_selection(bool active);
    void set_active_popup_paste(bool active);

    void block_played(int ms);

    // widgets
    Gnome::About *about;

    Gtk::HRuler *hruler1;
    View *view;
    Gtk::VBox *vbox1;
    Gtk::ScrolledWindow *scrolledwindow1;
    Gtk::FileSelection *fs, *fss, *fso;
    Gtk::Label *l1, *l3, *l5, *l7, *l8;
    DialogProject *dproject;
	  DialogExportWAV *dx_wav;
	  DialogExportMP3 *dx_mp3;
	  bool export_selection;
	  DialogProgress *export_progress;
	  DialogOptions *dialog_options;
	
    void show_config_dialog();
	
    Gtk::Menu *popup_menu_paste;
    Gtk::Menu *popup_menu;
    Gtk::Menu *popup_menu_selection;
    Gtk::MenuItem *popup_menu_selection_export_WAV;
    Gtk::MenuItem *popup_menu_selection_export_MP3;
    Gtk::MenuItem *popup_menu_paste_before;
    Gtk::MenuItem *popup_menu_paste_after;
    Gtk::MenuItem *popup_menu_selection_play_start;
    Gtk::MenuItem *popup_menu_selection_play_end;
//    Gtk::MenuItem *popup_menu_selection_change_start;
//    Gtk::MenuItem *popup_menu_selection_change_end;
    Gtk::MenuItem *popup_menu_selection_export_submenu;
    Gtk::MenuItem *popup_menu_block_select;
    Gtk::MenuItem *popup_menu_block_resize;
    Gtk::MenuItem *popup_menu_block_delete;

		
    int set_position(GdkEventMotion *);
    SigC::Connection timer_connection;

    BlockList project;        // project
    BlockList *clipboard;     // clipboard
    BlockList *selection;     // selection
    OSSdevice ossdev;
    int last_pos;
    bool displayed_position;
    bool select;              // selecting in progress
    int start_selection;      // start of selection in miliseconds
    int end_selection;        // end of selection in miliseconds

    int cursor_position_ms;   // position of cursor in ms -- for popup menu
    int project_totalMS, played_ms;

    bool close_canceled;      // closing of application is canceled in dialog 'Save changed project?'
    bool exit_after_save;

    GnomeConfig config;
    void read_config();
    void write_config();
    void get_config_from_dialog();

    // options
    Gdk_Color option_fg_color, option_bg_color;
    string option_mpg123_path, option_lame_path, option_sox_path, option_oss_device;
    bool option_use_wheel;
    int option_end_time;

    int option_exportwav_bps, option_exportwav_ch, option_exportwav_freq;
    bool option_mp3export_process, option_mp3export_strictequal;

};

#endif
