/***************************************************************************
              soundblock.h  -  base class for all sound formats
             ---------------------------------------------------
    begin                : Wed Feb 2 2000
    copyright            : (C) 2000 by Ladislav Slezk
    email                : slezak@penguin.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SOUNDBLOCK_H
#define SOUNDBLOCK_H

#include <sigc++/signal_system.h>
#include <gdk--/pixmap.h>
#include <glib--.h>

#include <vector>
#include <fstream>

#include "sounddevice.h"
#include "dialogprogress.h"


using namespace std;

/**Base class for blocks of sound
  *@author Ladislav Slezk
  */

class SoundBlock : public SigC::Object {

  public:

   	typedef enum {MODE_STEREO, MODE_JOINT_STEREO, MODE_DUAL, MODE_MONO} ChannelMode;

  	SoundBlock();
  	SoundBlock(const SoundBlock &src);
  	virtual ~SoundBlock();
  	
  	virtual bool init_play() = 0;                              // prepare block for playing
  	virtual bool play_block(SoundDevice *, string, int from_ms = -1, int to_ms = -1) = 0;        // play block to sound device
  	virtual bool is_changed() = 0;                          // something in block is changed
  	virtual bool scan_file(int, int) = 0;	                    // scan file
  	void display(Gdk_Pixmap *, const Gdk_GC &, int, int, int);	// display wave on pixmap
  	virtual int block_length() = 0;	                          //
  	virtual bool valid() = 0;				                          // is this soundblock valid?
  	virtual string block_info() = 0;				                  	// return info about this block
  	virtual bool assign_file(string) = 0;				              // assign file to this block
  	virtual bool assigned_file() = 0;				                  // is assigned
		virtual bool save_block_to_stream(ofstream &, string) = 0;   // save block
		virtual bool load_block_from_stream(ifstream &) = 0;      		// load block
  	virtual int num_samples() = 0;				                      // number of samples in block
    virtual int max_samples() = 0;                             // total number of samples in file
  	int get_frequency() {return freq;}				                      // get sampling frequency
  	int get_channels() {return channels;}	                    // return number of channels
  	virtual void stop_block() = 0;                             // stop playing
  	virtual void pause_block() = 0;                            // pause playing
  	virtual void unpause_block() = 0;                          // continue playing
  	virtual string get_type() = 0;                            	// get block type
  	string get_file_name() {return fileName;}              			// return file name
////////////////////////////  	
  	bool get_save_scanned_data() {return saveScannedData;}       // save scanned data status
  	void set_save_scanned_data(bool b) {saveScannedData = b;}    // set
  	virtual int get_begin_sample_offset() = 0;                   // get offset of first sample
  	virtual int get_end_sample_offset() = 0;                     //               last
  	virtual void set_begin_sample_offset(int) = 0;               // set offset of first sample
  	virtual void set_end_sample_offset(int) = 0;                 //               last
  	
  	virtual void export_WAV(ofstream &ostr, int chan, int freq, int bps, DialogProgress *dp,
  	                       string decoder_path, string conversion_path) = 0;
  	
		virtual void export_MP3(ofstream &ostr, string encoder_name,  string encoder_path, int bitrate,
		                     int frequency, ChannelMode channel_mode, bool crc, bool priv, bool original,
		                     bool copyright, vector<string> VBR_options, vector<string> encoder_options,
		                     bool always_process, bool strict_equal, DialogProgress *dp) = 0;
  	
    static string relativeFile(string absFileName, string absDirName);
  	
  protected:

    string blockName;       	// name of block
    gint32 freq;              // sampling frequency
    gint16 channels;          // number of sound channels

    bool changed;             // block is changed

    string fileName;          // sound filename
    ifstream file;
		G_Date fileDateTime;			// modificaction time

    gint32 begin;             // sound data start
    gint32 end;               // end of sound data
    gint32 beginMax;          // maximum positions
    gint32 endMax;

 		SoundDevice *sdev;        // play block into this sound device

    bool assigned;            // block has assigned file
    bool playing;    					// block is playing
    bool paused;     					// block is paused

    int play_from_ms, play_to_ms;		// play start/end of block

    vector<gint16> scannedData;     // array of scanned data
		bool saveScannedData;           // save array into project

  public:
    // signals
//		SigC::Signal1<void, int> playedMiliSec;       // nuber of played miliseconds
		SigC::Signal0<void> block_done;                // playing done
		SigC::Signal0<void> scan_done;                 // scan done
		SigC::Signal0<void> block_changed;            // sound block changed
		SigC::Signal0<void> export_done;              // export done
		
};

#endif
