/***************************************************************************
                      utils.cpp  -  some useful functions
                     -------------------------------------
    begin                : Sun Feb 11 2001
    copyright            : (C) 2001 by Ladislav Slezk
    email                : slezak@penguin.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "utils.h"

#include <strstream.h>
#include <glib.h>

#include <cstdio>

Utils::Utils(){
}

Utils::~Utils(){
}

string Utils::ms2str(int ms) {
  int sec = (ms / 1000) % 60;
  int min = ms / 1000 / 60;
  ms = ms % 1000;

  char buffer[32];

  sprintf(buffer, "%02d:%02d:%02d", min, sec, ms / 10);

  return string(buffer);

//  strstream ss;
//  ss.fill('0');
//  ss.width(2);
//  ss << min << ':';
//  ss.fill('0');
//  ss.width(2);
//  ss << sec << '.';
//  ss.fill('0');
//  ss.width(2);
//  ss << ms/10;
//
//  string s;
//  ss >> s;
//
//  return s;
}

string Utils::relativeFile(string absFileName, string absDirName) {
  if (absFileName.size() == 0 || absDirName.size() == 0)
    return string("");

  // delete equal beginnig
  while (absFileName[0] == absDirName[0]) {
    absFileName.erase(0, 1);
    absDirName.erase(0, 1);
  }

  string result;

  if (absDirName.size() == 0) result = ".";
  else {
    result = "../";

    int pos = absDirName.find('/');

    while (pos > 0) {
      result = result + "../";
      absDirName[pos] = '#';
      pos = absDirName.find('/');
    }
  }

  result = result + absFileName;

  return result;
}

string Utils::absoluteFile(string relFileName, string absDirName) {
  if (relFileName.empty() || absDirName.empty()) return string("");

  while (relFileName.size() > 0) {

    if (relFileName[0] != '.') {
      if (absDirName.empty()) absDirName = "/";
      int pos = relFileName.find('/');              // search next '/'

      if (pos < 0) {                       // '/' not found, relFileName is name without any path
        absDirName += relFileName;         // copy filename
        return absDirName;
      }
      else {
        string dir(relFileName, 0 , pos);             // get start
        relFileName.erase(relFileName.begin(), relFileName.begin() + pos + 1);
        absDirName += dir;
        absDirName += "/";
      }
    }
    else
      if (relFileName.size() > 2 && relFileName[0] == '.' && relFileName[1] == '/')   // relative name starts './'
        relFileName.erase(relFileName.begin(), relFileName.begin() + 1);              // only delete from relative name
      else
        if (relFileName.size() > 3 && relFileName[0] == '.' && relFileName[1] == '.' && relFileName[2] == '/') {
          // relative name starts '../'
          absDirName.erase(absDirName.begin() + absDirName.rfind('/'), absDirName.end());  // move to root
          if (absDirName.empty()) absDirName = "/";
          relFileName.erase(relFileName.begin(), relFileName.begin() + 3);
        }
  }

  return absDirName;
}

bool Utils::write_WAV_header(ostream &os, int ch, int f, int bps) {

  os << "RIFF";      // RIFF format
  os << "0000";      // length
  os << "WAVE";      // WAVE audio
  os << "fmt ";      // format chunk

  guint32 fmt_len = 16;
  os.write(&fmt_len, sizeof(guint32));   // length of format chunk is 16 bytes

  guint16 format = 1;
  os.write(&format, sizeof(guint16));    // PCM audio format

  guint16 chan = ch;
  os.write(&chan, sizeof(guint16));      // number of channels

  guint32 fr = f;
  os.write(&fr, sizeof(guint32));        // sampling frequency

  guint32 avg = f * ch * (bps/8);
  os.write(&avg, sizeof(guint32));       // average bytes per second

  guint16 s = ch * (bps / 8);
  os.write(&s, sizeof(guint16));         // size of one sample in bytes

  guint16 bps_2 = bps;
  os.write(&bps_2, sizeof(guint16));     // bits per sample

  os << "data";      // data chunk
  os << "0000";      // length of data

  return true;
}

bool Utils::write_WAV_header_len(ostream &os) {
  os.seekp(0, ios::end);
  int length = os.tellp();

  os.seekp(4, ios::beg);
  guint32 len_riff = length - 8;
  os.write(&len_riff, sizeof(guint32));       // write total length

  os.seekp(40, ios::beg);
  guint32 len_data = length - 44;
  os.write(&len_data, sizeof(guint32));       // write data length

  return true;
}

bool Utils::write_ID3_tag(ostream &os, DialogTagEditor *id3) {
  if (id3 == 0) return false;     // no ID3 tag passed

  // get ID3 tag properties
  string title = id3->get_title();
  while (title.size() < 30) {
    title.append(" ");
  }

  string artist = id3->get_artist();
  while (artist.size() < 30) {
    artist.append(" ");
  }

  string album = id3->get_album();
  while (album.size() < 30) {
    album.append(" ");
  }

  string year = id3->get_year();
  while (year.size() < 4) {
    year.append(" ");
  }

  string comment = id3->get_comment();
  while (comment.size() < 30) {
    comment.append(" ");
  }

  // write ID3 tag identification
  os << "TAG";

  // write tag attributes
  os << title;
  os << artist;
  os << album;
  os << year;
  os << comment;
  os << id3->get_genre();

  return os;
}
